/*

  This file is part of DIG Library.
  DIG Library is a free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 2 of the License,
  or (at your option) any later version.
  DIG Library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  General Public License for more details.
  You should have received a copy of the GNU General Public License
  along with Foobar; if not, write to the Free Software Foundation,
  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

*/

// DIGFile.cpp,v 1.20 2007/06/14 17:10:50 swr Exp

//defines added to enable large file support
//jklukowska, 06/07/2007
#ifndef _LARGEFILE_SOURCE
	#define _LARGEFILE_SOURCE     /* enable large file support  */
#endif
#ifndef _FILE_OFFSET_BITS
	#define _FILE_OFFSET_BITS 64     /* enable large file support  */
#endif

// bug 226 - needed to compile this file on cygwin - swr - 6/14/07
#undef __STRICT_ANSI__

//#include <iostream>
#include <cstring>
#include <cstdlib>
#include <cstdio>
#include <cmath>

#include <xercesc/framework/StdOutFormatTarget.hpp>
#include <xercesc/framework/LocalFileFormatTarget.hpp>
#include <xercesc/framework/MemBufFormatTarget.hpp>

//#include <xercesc/util/XMLDouble.hpp>


//#include "DIGFile.h"
#include <DIGFile/DIGFile.h>

#define DIGFILE_DEBUG_LEVEL 0
        

const char* DIGEndianStr[DIGEndianSize] = {
  "little", 
  "big"
};

const char* DIGValueTypeStr[DIGValueTypeSize] = {
  "real", 
  "complex"
}; 

const char* DIGDataTypeStr[DIGDataTypeSize] = {
  "bool", 
  "unsigned char", 
  "char", 
  "unsigned short", 
  "short", 
  "unsigned int", 
  "int", 
  "float", 
  "double"
}; 

const char* DIGDataFormatStr[DIGDataFormatSize] = {
  "ASCII", 
  "binary"
};

const char* DIGGridStr[DIGGridSize] = {
  "SC", 
  "FCC1", 
  "FCC2", 
  "BCC1", 
  "BCC2", 
  "BCC3", 
  "BCC4", 
  "HEX1", 
  "HEX2"
};

const char* DIGBasisStr[DIGBasisSize] = {
  "voronoi", 
  "blob"
};

const char* DIGUnitStr[DIGUnitSize] = {
  "unspecified", 
  "m", 
  "cm", 
  "mm", 
  "nm", 
  "micron", 
  "A", 
  "other"
};

static char DIGEmptyStr[1] = "";

bool DIGFile::XMLStringsInitialized = false;

const XMLCh* DIGFile::RootXMLStr;

const XMLCh* DIGFile::MainHeaderXMLStr;

const XMLCh* DIGFile::TitleXMLStr;
const XMLCh* DIGFile::TypeXMLStr;
const XMLCh* DIGFile::ChannelsXMLStr;
const XMLCh* DIGFile::NumberOfArraysXMLStr;
const XMLCh* DIGFile::NumberOfArraySetsXMLStr;
const XMLCh* DIGFile::EndianXMLStr;
const XMLCh* DIGFile::ValueTypeXMLStr;
const XMLCh* DIGFile::DataTypeXMLStr;
const XMLCh* DIGFile::DataFormatXMLStr;
const XMLCh* DIGFile::GridTypeXMLStr;
const XMLCh* DIGFile::BasisFunctionXMLStr;
const XMLCh* DIGFile::UnitXMLStr;
const XMLCh* DIGFile::OtherUnitXMLStr;
const XMLCh* DIGFile::CommentsXMLStr;
const XMLCh* DIGFile::DimensionsXMLStr;

const XMLCh* DIGFile::DimmensionXXMLStr;
const XMLCh* DIGFile::DimmensionYXMLStr;      
const XMLCh* DIGFile::DimmensionZXMLStr;

const XMLCh* DIGFile::SamplingRateXXMLStr;
const XMLCh* DIGFile::SamplingRateYXMLStr;
const XMLCh* DIGFile::SamplingRateZXMLStr;

const XMLCh* DIGFile::ArraySetHeaderXMLStr;

//const XMLCh* DIGFile::TypeXMLStr;
//const XMLCh* DIGFile::TitleXMLStr;
const XMLCh* DIGFile::ParametersXMLStr;

const XMLCh* DIGFile::ArrayHeaderXMLStr;
const XMLCh* DIGFile::EnumerationNumberXMLStr;


class DOMPrintErrorHandler : public DOMErrorHandler
{
public:

  DOMPrintErrorHandler(){};
  ~DOMPrintErrorHandler(){};

  /** @name The error handler interface */
  bool handleError(const DOMError& domError);
  void resetErrors(){};

private :
  /* Unimplemented constructors and operators */ 
  DOMPrintErrorHandler(const DOMErrorHandler&);
  void operator=(const DOMErrorHandler&);
    
};

bool DOMPrintErrorHandler::handleError(const DOMError &domError)
{
  // Display whatever error message passed from the serializer
  char *msg = XMLString::transcode(domError.getMessage());
  std::cout<<msg<<std::endl;
  delete[] msg;

  // Instructs the serializer to continue serialization if possible.
  return true;
}


DIGFile::DIGFile()
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::DIGFile\n");
#endif

  MainHeaderWritten = false;
  ArraySetOpen = false;
  //ArrayCounter = 0;

  NoOfSets = 0;
  NoOfArrays = 0;

  Dirty = false;
  File = NULL;

  // ??????????????????????????
	// Initialization

#if DIGFILE_DEBUG_LEVEL > 1
  printf("Initializing XML parser\n");
#endif

	try {
		XMLPlatformUtils::Initialize();
	}
	catch(const XMLException &toCatch) {
		std::cerr << "Error during Xerces-c Initialization.\n"
			<< "  Exception message:"
			<< StrX(toCatch.getMessage()) << std::endl;
		//return 1;
	}
  
  if(!XMLStringsInitialized) {

    RootXMLStr               = XMLString::transcode("DIG_DATA");

    MainHeaderXMLStr         = XMLString::transcode("main_header");

    TitleXMLStr              = XMLString::transcode("title");
    TypeXMLStr               = XMLString::transcode("type");
    ChannelsXMLStr           = XMLString::transcode("channels");
    NumberOfArraysXMLStr     = XMLString::transcode("number_of_arrays");
    NumberOfArraySetsXMLStr  = XMLString::transcode("number_of_array_sets");
    EndianXMLStr             = XMLString::transcode("endian");
    ValueTypeXMLStr          = XMLString::transcode("value_type");
    DataTypeXMLStr           = XMLString::transcode("data_type");
    DataFormatXMLStr         = XMLString::transcode("data_format");
    GridTypeXMLStr           = XMLString::transcode("grid_type");
    BasisFunctionXMLStr      = XMLString::transcode("basis_function");
    UnitXMLStr               = XMLString::transcode("unit");
    OtherUnitXMLStr          = XMLString::transcode("other_unit");
    CommentsXMLStr           = XMLString::transcode("comments");
    DimensionsXMLStr         = XMLString::transcode("dimensions");

    DimmensionXXMLStr        = XMLString::transcode("x");
    DimmensionYXMLStr        = XMLString::transcode("y");
    DimmensionZXMLStr        = XMLString::transcode("z");

    SamplingRateXXMLStr      = XMLString::transcode("sampling_rate_x");
    SamplingRateYXMLStr      = XMLString::transcode("sampling_rate_y");
    SamplingRateZXMLStr      = XMLString::transcode("sampling_rate_z");

    ArraySetHeaderXMLStr     = XMLString::transcode("array_set_header");

    // TypeXMLStr               = XMLString::transcode("type");
    // TitleXMLStr              = XMLString::transcode("title");
    ParametersXMLStr         = XMLString::transcode("parameters");

    ArrayHeaderXMLStr        = XMLString::transcode("array_header");
    EnumerationNumberXMLStr  = XMLString::transcode("enumeration_number");

    XMLStringsInitialized = true;
  }
}

int DIGFile::FindSrting(char* String, FILE* File)
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::FindSrting\n");
#endif

  int i;
  char ch;
  int offset = 0;

  // get legth of the string
  int len = strlen(String);
  
  // create input buffer
  char* Buffer = new char[len];
  int head = 0; 
  int tail = 0;

  for(;;) {
    // find match for first character in the string
    while((ch = getc(File)) != String[0] && (ch != EOF)) {
      offset++;
    }

    if(ch == EOF) {
      return -1; // not found
    }

    // put first file character into the buffer
    Buffer[0] = ch;
    tail = 1;
    int count = 1;

    do {
      // fill Buffer with file characters
      for(i = count; i < len; i++) {
        if((ch = getc(File)) == EOF) {
          return -1;
        }
        Buffer[tail] = ch;
        tail = (tail + 1) % len;
      }

      count = len;

      // compare
      int ptr = head;

      for(i = 1; i < len; i++) {

        ptr = (ptr + 1) % len;

        if(Buffer[ptr] != String[i]) {
          break; // found first difference
        }
      }

      if(i == len) { // full match
        return offset;
      }

      // move head to the next match for first character
      do {
        count--;
        head = (head + 1) % len;
        offset++;

        if(Buffer[head] == String[0]) {
          break; // found match for first character in the string
        }
      } 
      while(count > 0);
    }
    while(count > 0);
  }
}


int DIGFile::GetXMLHeader(char** XMLBuff)
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::GetXMLHeader\n");
#endif

  static char TerminationString[] = "</DIG_DATA>";
  
#if DIGFILE_DEBUG_LEVEL > 1
  printf("Searching for XML termnation\n");
#endif

  // search for end of XML
  if((XMLHeaderLength = FindSrting(TerminationString, File)) == -1) {
#if DIGFILE_DEBUG_LEVEL > 0
    printf("** Error: missing XML termnation\n");
#endif
    return -1; // error missing XML termnation
  }

  // add length of termnating string and 1 for the following LF
  XMLHeaderLength += strlen(TerminationString) + 1;

#if DIGFILE_DEBUG_LEVEL > 1
  printf("Reading XML Header\n");
#endif

  // read XML to memory
  *XMLBuff = new char[XMLHeaderLength+1];

  rewind(File);

  if(fread((void*) *XMLBuff, 1, XMLHeaderLength, File) != (unsigned int) XMLHeaderLength) {

#if DIGFILE_DEBUG_LEVEL > 0
    printf("** Error: reading file\n");
#endif
    return -1; // error reading file
  }

  (*XMLBuff)[XMLHeaderLength] = 0;

  return 0;
}


// open existing file

int DIGFile::Open(const char* FileName)
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::Open\n");
#endif

  char* XMLBuffer;

  if(File != NULL) {

#if DIGFILE_DEBUG_LEVEL > 0
    printf("** Error: DIG File already open\n");
#endif
    return -1; // DIG File already open
  }

#if DIGFILE_DEBUG_LEVEL > 1
  printf("Opening DIG file for reading\n");
#endif

  // try to open DIG File for reading
  if((File = fopen(FileName, "r+b")) == NULL) {

#if DIGFILE_DEBUG_LEVEL > 0
    printf("** Error: unable to open DIG File\n");
#endif
    return -1; // unable to open DIG File
  }

#if DIGFILE_DEBUG_LEVEL > 1
    printf("Loading XML header\n");
#endif

  // load XML header
  if(GetXMLHeader(&XMLBuffer) != 0) {
#if DIGFILE_DEBUG_LEVEL > 0
    printf("** Error: invalid DIG File (missing XML header\n");
#endif
    return -2; // invalid DIG File (missing XML header)
  }

#if DIGFILE_DEBUG_LEVEL > 1
  printf("Parsing XML Header\n");
#endif

  // parse XML and load structures
  if(ParseXML(XMLBuffer) != 0) {
#if DIGFILE_DEBUG_LEVEL > 0
    printf("** Error: unable to parse XML header\n");
#endif
    return -3; // unable to parse XML header
  }

  NoOfArrayItems = ComputeArrayNoOfItems();
  ArrayBufferSize = ComputeArrayBufferSize();
  Dirty = false;

  return 0;
}


// open new file

int DIGFile::Open(
  const char*           pFileName,
  const char*           pSchema,
 	const char*           pTitle,
 	const char*           pType,
	unsigned int          pChannels,
	//DIGEndian             pEndian,
	DIGValueType          pValueType,
	DIGDataType           pDataType,
	DIGDataFormat         pDataFormat,
	DIGGrid               pGrid,
	DIGBasis              pBasis,
	DIGUnit               pUnit,
	const DIGDimensions*  pDimensions,
	const DIGSampling*    pSamplingX,
	const DIGSampling*    pSamplingY,
	const DIGSampling*    pSamplingZ,
	const char*           pComment,
	const char*           pOtherUnit
)
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::Open\n");
#endif

  if(File != NULL) {
#if DIGFILE_DEBUG_LEVEL > 0
    printf("** Error: DIG File already open\n");
#endif
    return -1; // DIG File already open
  }

  if(SchemaName.Set(pSchema) != 0) {
#if DIGFILE_DEBUG_LEVEL > 0
    printf("** Error: \n");
#endif
    return -1; // 
  }

	// open temp file for data
  TmpDataFile = tmpfile();

#if DIGFILE_DEBUG_LEVEL > 1
  printf("Opening DIG File\n");
#endif

  if((File = fopen(pFileName, "w+b")) == NULL) {
#if DIGFILE_DEBUG_LEVEL > 0
    printf("** Error: unable to open file\n");
#endif
    return -1; // unable to open file
  }	

  MainHeader.Title.Set(pTitle);
  MainHeader.Type.Set(pType);
  MainHeader.Channels = pChannels;
  //MainHeader.number_of_arrays = ;
  //MainHeader.number_of_sets =;
  //MainHeader.Endian = pEndian;
  MainHeader.ValueType = pValueType;
  MainHeader.DataType = pDataType;
  MainHeader.DataFormat = pDataFormat;
  MainHeader.Grid = pGrid;
  MainHeader.Basis = pBasis;
  MainHeader.Unit = pUnit;
	MainHeader.OtherUnit.Set(pOtherUnit);
  MainHeader.Dimensions.x = pDimensions->x;
  MainHeader.Dimensions.y = pDimensions->y;
  MainHeader.Dimensions.z = pDimensions->z;
  MainHeader.SamplingX.x = pSamplingX->x;
  MainHeader.SamplingX.y = pSamplingX->y;
  MainHeader.SamplingX.z = pSamplingX->z;
  MainHeader.SamplingY.x = pSamplingY->x;
  MainHeader.SamplingY.y = pSamplingY->y;
  MainHeader.SamplingY.z = pSamplingY->z;
  MainHeader.SamplingZ.x = pSamplingZ->x;
  MainHeader.SamplingZ.y = pSamplingZ->y;
  MainHeader.SamplingZ.z = pSamplingZ->z;

  MainHeader.Comment.Set(pComment);

  Dirty = true;

  NoOfArrayItems = ComputeArrayNoOfItems();
  ArrayBufferSize = ComputeArrayBufferSize();


  XMLCh tempStr[100];

  XMLString::transcode("Core", tempStr, 99);

  impl =  DOMImplementationRegistry::getDOMImplementation(tempStr);

  return 0;
}


int DIGFile::SetComment(const char* pComment)
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::SetComment\n");
#endif

  MainHeader.Comment.Set(pComment);
  return 0;
}


int DIGFile::SetTitle(const char* pTitle)
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::SetTitle\n");
#endif

  MainHeader.Title.Set(pTitle);
  return 0;
}


int DIGFile::GetTitle(const char** pTitle)
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::GetTitle\n");
#endif

  *pTitle = MainHeader.Title.Get();
  return 0;
}


int DIGFile::GetType(const char** pType)
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::GetType\n");
#endif

  *pType = MainHeader.Type.Get();
  return 0;
}


int DIGFile::GetChannels(unsigned int* pChannels)
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::GetChannels\n");
#endif

  *pChannels = MainHeader.Channels;
  return 0;
}


int DIGFile::GetEndian(DIGEndian* pEndian)
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::GetEndian\n");
#endif

  *pEndian = Endian;
  return 0;
}


int DIGFile::GetValueType(DIGValueType* pValueType)
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::GetValueType\n");
#endif

  *pValueType = MainHeader.ValueType;
  return 0;
}


int DIGFile::GetDataType(DIGDataType* pDataType)
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::GetDataType\n");
#endif

  *pDataType = MainHeader.DataType;
  return 0;
}


int DIGFile::GetDataFormat(DIGDataFormat* pDataFormat)
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::GetDataFormat\n");
#endif

  *pDataFormat = MainHeader.DataFormat;
  return 0;
}


int DIGFile::GetGrid(DIGGrid* pGrid)
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::GetGrid\n");
#endif

  *pGrid = MainHeader.Grid;
  return 0;
}


int DIGFile::GetBasis(DIGBasis* pBasis)
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::GetBasis\n");
#endif

  *pBasis = MainHeader.Basis;
  return 0;
}


int DIGFile::GetUnit(DIGUnit* pUnit)
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::GetUnit\n");
#endif

  *pUnit = MainHeader.Unit;
  return 0;
}


int DIGFile::GetOtherUnit(const char** pOtherUnit)
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::GetOtherUnit\n");
#endif

  *pOtherUnit = MainHeader.OtherUnit.Get();
  return 0;
}


int DIGFile::GetDimensions(const DIGDimensions** pDimensions)
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::GetDimensions\n");
#endif

  *pDimensions = &(MainHeader.Dimensions);
  return 0;
}


int DIGFile::GetSamplingX(const DIGSampling** pSampling)
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::GetSamplingX\n");
#endif

  *pSampling = &(MainHeader.SamplingX);
  return 0;
}


int DIGFile::GetSamplingY(const DIGSampling** pSampling)
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::GetSamplingY\n");
#endif

  *pSampling = &(MainHeader.SamplingY);
  return 0;
}


int DIGFile::GetSamplingZ(const DIGSampling** pSampling)
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::GetSamplingZ\n");
#endif

  *pSampling = &(MainHeader.SamplingZ);
  return 0;
}


int  DIGFile::GetComment(const char** pComment)
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::GetComment\n");
#endif

  *pComment = MainHeader.Comment.Get();
  return 0;
}


unsigned int DIGFile::ComputeArrayNoOfItems()
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::ComputeArrayNoOfItems\n");
#endif

  int Count = MainHeader.Channels * 
              MainHeader.Dimensions.x *
              MainHeader.Dimensions.y *
              MainHeader.Dimensions.z;

  switch(MainHeader.ValueType) {
  case DIGValueType_REAL:
    return Count;

  case DIGValueType_COMPLEX:
    return Count * 2;

  default:
    return 0; // error
  }
}


int DIGFile::GetArrayNoOfItems(unsigned int* pArrayNoOfItems)
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::GetArrayNoOfItems\n");
#endif

  *pArrayNoOfItems = NoOfArrayItems;
  return 0;
}


unsigned int DIGFile::ComputeArrayBufferSize()
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::ComputeArrayBufferSize\n");
#endif

  unsigned int Size = NoOfArrayItems;

  switch(MainHeader.DataType) {

  case DIGDataType_BOOL:
  case DIGDataType_UCHAR:
  case DIGDataType_CHAR:
    return Size;

  case DIGDataType_USHORT:
  case DIGDataType_SHORT:
    return Size * sizeof(short);

  case DIGDataType_UINT:
  case DIGDataType_INT:
    return Size * sizeof(int);

  case DIGDataType_FLOAT:
    return Size * sizeof(float);

  case DIGDataType_DOUBLE:
    return Size * sizeof(double);

  default:
    return 0; // error
  }
}


int DIGFile::GetArrayBufferSize(unsigned int* pArrayBufferSize)
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::GetArrayBufferSize\n");
#endif

  *pArrayBufferSize = ArrayBufferSize;
  return 0;
}


#define DIG_FILE_BUFFER_SIZE 1024

int DIGFile::Close()
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::Close\n");
#endif

  if(Dirty) {

    // Append data to ouput file (Header is already there)

    char DataBuffer[DIG_FILE_BUFFER_SIZE];
    int  ReadByteCount;

    if(WriteXMLHeader() != 0) {

#if DIGFILE_DEBUG_LEVEL > 0
    printf("** Error: unable to write XML header\n");
#endif
      return -1;
    }

    // copy data from temp file to output file
    rewind(TmpDataFile);
    while((ReadByteCount = fread(DataBuffer, 1, DIG_FILE_BUFFER_SIZE, TmpDataFile)) > 0) {
      if(ferror(TmpDataFile) != 0) {

#if DIGFILE_DEBUG_LEVEL > 0
    printf("** Error: error reading temp file\n");
#endif
        return -2; // error reading temp file
      }

      if(fwrite(DataBuffer, 1, ReadByteCount, File) != (unsigned int) ReadByteCount) {

#if DIGFILE_DEBUG_LEVEL > 0
    printf("** Error: error writing  file\n");
#endif
        return -3; // error writing  file
      }
    }
  }

  ArraySets.Release();

  if(File == NULL) {

#if DIGFILE_DEBUG_LEVEL > 0
    printf("** Error: attempt to close unopen file\n");
#endif
    return -2; // attempt to close unopen file
  }

  fclose(File);

  File = NULL;

  return 0;
}


int DIGFile::AppendArraySet(const char* pType, const char* pTitle, const char* pParameters, const char* pComments)
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::AppendArraySet\n");
#endif

  // create new array set header
  DIGFileArraySetHeader* SetHeader = new DIGFileArraySetHeader(pType, pTitle, pParameters, pComments);

  // create new array set
  CurrenArraySet = new ArraySet;

  // attach array set header
  CurrenArraySet->ArraySetHeader = SetHeader;
  CurrenArraySet->FirstArrayOffset = NoOfArrays;

  // append array set to array set list
  ArraySets.Append(CurrenArraySet);

  NoOfSets = ArraySets.getLength();

  Dirty = true;

  return 0;
}


int DIGFile::AppendArray(unsigned int pEnumNo, const char* pComment, const void* pBuffer)
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::AppendArray\n");
#endif

  if(CurrenArraySet == NULL) {

#if DIGFILE_DEBUG_LEVEL > 0
    printf("** Error: no array set selected\n");
#endif
    return -1; // no array set selected
  }

  DIGFileArrayHeader* ArrayHeader = new DIGFileArrayHeader(pEnumNo, pComment);

  // append to array list
  CurrenArraySet->Append(ArrayHeader);

  // append data to temp file
  if(WriteArrayData(TmpDataFile, pBuffer) != 0) {

#if DIGFILE_DEBUG_LEVEL > 0
    printf("** Error: writing data\n");
#endif
    return -2;
  }

  NoOfArrays++;

  Dirty = true;

  return 0;
}


// returns number of array sets in DIG file

int DIGFile::GetNoOfArraySets(unsigned int* pNoOfArraySets)
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::GetNoOfArraySets\n");
#endif

  *pNoOfArraySets = ArraySets.getLength();
  return 0;
}


// returns number of arrays in current array set

int DIGFile::GetNoOfArrays(unsigned int* pNoOfArrays)
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::GetNoOfArrays\n");
#endif

  if(CurrenArraySet == NULL) {

#if DIGFILE_DEBUG_LEVEL > 0
    printf("** Error: no array set selected\n");
#endif
    return -1; // no array set selected
  }
  *pNoOfArrays = CurrenArraySet->getLength();
  return 0;
}

// returns array set header of selected array set
/*
int DIGFile::GetArraySetHeader(DIGFileArraySetHeader** SetHeader)
{
  *SetHeader = CurrenArraySet->ArraySetHeader;
  return 0;
}
*/

// returns array header of selected array

/*
int DIGFile::GetArrayHeader(DIGFileArrayHeader** ArrayHeader)
{
  *ArrayHeader = CurrentArrayHeader;
  return 0;
}
*/

int DIGFile::GetArraySetType(const char** pType)
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::GetArraySetType\n");
#endif

  if(CurrenArraySet == NULL) {

#if DIGFILE_DEBUG_LEVEL > 0
    printf("** Error: no array set selected\n");
#endif
    return -1; // no array set selected
  }

  *pType = CurrenArraySet->ArraySetHeader->Type.Get();
  return 0;
}


int DIGFile::GetArraySetTitle(const char** pTitle) 
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::GetArraySetTitle\n");
#endif

  if(CurrenArraySet == NULL) {

#if DIGFILE_DEBUG_LEVEL > 0
    printf("** Error: no array set selected\n");
#endif
    return -1; // no array set selected
  }

  *pTitle = CurrenArraySet->ArraySetHeader->Title.Get();
  return 0;
}


int DIGFile::GetArraySetParameters(const char** pParameters)
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::GetArraySetParameters\n");
#endif

  if(CurrenArraySet == NULL) {

#if DIGFILE_DEBUG_LEVEL > 0
    printf("** Error: no array set selected\n");
#endif
    return -1; // no array set selected
  }

  *pParameters = CurrenArraySet->ArraySetHeader->Parameters.Get();
  return 0;
}


int DIGFile::GetArraySetComment(const char** pComment)
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::GetArraySetComment\n");
#endif

  if(CurrenArraySet == NULL) {

#if DIGFILE_DEBUG_LEVEL > 0
    printf("** Error: no array set selected\n");
#endif
    return -1; // no array set selected
  }

  *pComment = CurrenArraySet->ArraySetHeader->Comment.Get();
  return 0;
}


int DIGFile::GetArrayEnumNo(unsigned int* pEnumNo)
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::GetArrayEnumNo\n");
#endif

  if(CurrenArraySet == NULL) {

#if DIGFILE_DEBUG_LEVEL > 0
    printf("** Error: no array set selected\n");
#endif
    return -1; // no array set selected
  }

  *pEnumNo = CurrentArrayHeader->EnumNo;
  return 0;
}


int DIGFile::GetArrayComment(const char** pComment)
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::GetArrayComment\n");
#endif

  if(CurrenArraySet == NULL) {

#if DIGFILE_DEBUG_LEVEL > 0
    printf("** Error: no array set selected\n");
#endif
    return -1; // no array set selected
  }

  *pComment = CurrentArrayHeader->Comment.Get();
  return 0;
}


// selects array set

int DIGFile::SelectArraySet(unsigned int No)
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::SelectArraySet\n");
#endif

  CurrentArrayHeader = NULL;
  return ArraySets.getArraySet(&CurrenArraySet, No);
}


// selects array from current array set

int DIGFile::SelectArray(unsigned int No)
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::SelectArray\n");
#endif

  if(CurrenArraySet == NULL) {

#if DIGFILE_DEBUG_LEVEL > 0
    printf("** Error: no array set selected\n");
#endif
    return -1;
  }

  CurrenArrayIndex = CurrenArraySet->FirstArrayOffset + No;

  return CurrenArraySet->getArrayHeader(&CurrentArrayHeader, No);
}


// returns data of selected array

int DIGFile::GetArrayData(void* ArrayData)
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::GetArrayData\n");
#endif

  if(CurrenArraySet == NULL) {

#if DIGFILE_DEBUG_LEVEL > 0
    printf("** Error: no array set selected\n");
#endif
    return -1;
  }

  //type of CurrentArrayOffset changed from unsigned int to off_t
  //to enable large file support (>2GB)
  //jklukowska, 06/07/2007
  off_t CurrenArrayOffset = static_cast <off_t>(XMLHeaderLength) + 
  			static_cast <off_t> (CurrenArrayIndex) * static_cast <off_t>(ArrayBufferSize);
  //unsigned int CurrenArrayOffset = XMLHeaderLength + CurrenArrayIndex * ArrayBufferSize;

  //fseek() changed to fseeko() to enable large file support (>2GB)
  //jklukowska, 06/07/2007
  if(fseeko(File, CurrenArrayOffset, SEEK_SET) != 0) {
  //if(fseek(File, CurrenArrayOffset, SEEK_SET) != 0) {

#if DIGFILE_DEBUG_LEVEL > 0
    printf("** Error: seek error\n");
#endif
    return -2;
  }

  switch(MainHeader.DataFormat) {
  case DIGDataFormat_ASCII:

#if DIGFILE_DEBUG_LEVEL > 0
    printf("** Error: Reading ASCII DIGFiles not supported\n");
#endif
    return -1; // Reading ASCII DIGFiles not supported
    /*
    switch(MainHeader.DataType) {

    case DIGDataType_BOOL:
      for(i = 0; i < NoOfArrayItems; i++) {
        if(((unsigned char*) ArrayData)[i] == 0) {
          if(fprintf(f, " 0") < 0) {
            return -1; // error
          }
        }
        else {
          if(fprintf(f, " 1") < 0) {
            return -1; // error
          }
        }
      }
      break;

    case DIGDataType_UCHAR:
      for(i = 0; i < NoOfArrayItems; i++) {
        if(fprintf(f, " %u", ((unsigned char*) ArrayData)[i]) < 0) {
          return -1; // error
        }
      }
      break;

    case DIGDataType_CHAR:
      for(i = 0; i < NoOfArrayItems; i++) {
        if(fprintf(f, " %i", ((char*) ArrayData)[i]) < 0) {
          return -1; // error
        }
      }
      break;

    case DIGDataType_USHORT:
      for(i = 0; i < NoOfArrayItems; i++) {
        if(fprintf(f, " %u", ((unsigned short*) ArrayData)[i]) < 0) {
          return -1; // error
        }
      }
      break;

    case DIGDataType_SHORT:
      for(i = 0; i < NoOfArrayItems; i++) {
        if(fprintf(f, " %i", ((short*) ArrayData)[i]) < 0) {
          return -1; // error
        }
      }
      break;

    case DIGDataType_UINT:
      for(i = 0; i < NoOfArrayItems; i++) {
        if(fprintf(f, " %u", ((unsigned int*) ArrayData)[i]) < 0) {
          return -1; // error
        }
      }
      break;

    case DIGDataType_INT:
      for(i = 0; i < NoOfArrayItems; i++) {
        if(fprintf(f, " %i", ((int*) ArrayData)[i]) < 0) {
          return -1; // error
        }
      }
      break;

    case DIGDataType_FLOAT:
      for(i = 0; i < NoOfArrayItems; i++) {
        if(fprintf(f, " %f", ((double) ((float*) ArrayData)[i])) < 0) {
          return -1; // error
        }
      }
      break;

    case DIGDataType_DOUBLE:
      for(i = 0; i < NoOfArrayItems; i++) {
        if(fprintf(f, " %f", ((double*) ArrayData)[i]) < 0) {
          return -1; // error
        }
      }
      break;

    default:
      return -2; // error
    }
    */
    break;

  case DIGDataFormat_BINARY:

    // read data bytes from the file
    if(fread(ArrayData, sizeof(char), ArrayBufferSize, File) != ArrayBufferSize) {

#if DIGFILE_DEBUG_LEVEL > 0
    printf("** Error: Reading binary data\n");
#endif
      return -3;
    }

    // if file endian different than current machine flip bytes
    if(Endian != DIG_ENDIAN) { 

      unsigned int i;

      switch(MainHeader.DataType) {

      // one byte types
      case DIGDataType_BOOL:
      case DIGDataType_UCHAR:
      case DIGDataType_CHAR:
        // bytes formats are in right order
        break;

      // 2 byte types
      case DIGDataType_USHORT:
      case DIGDataType_SHORT:
        for(i = 0; i < NoOfArrayItems; i++) {
          short val;

          *((char*) &val)       = ((char*) ArrayData)[2 * i + 1];
          *(((char*) &val) + 1) = ((char*) ArrayData)[2 * i];

          ((short*) ArrayData)[i] = val;
        }
        break;

      // four byte types
      case DIGDataType_UINT:
      case DIGDataType_INT:
      case DIGDataType_FLOAT:
        for(i = 0; i < NoOfArrayItems; i++) {
          int val;

          *((char*) &val)       = ((char*) ArrayData)[4 * i + 3];
          *(((char*) &val) + 1) = ((char*) ArrayData)[4 * i + 2];
          *(((char*) &val) + 2) = ((char*) ArrayData)[4 * i + 1];
          *(((char*) &val) + 3) = ((char*) ArrayData)[4 * i];

          ((int*) ArrayData)[i] = val;
        }
        break;

      // eight byte types
      case DIGDataType_DOUBLE:
        for(i = 0; i < NoOfArrayItems; i++) {
          double val;

          *((char*) &val)       = ((char*) ArrayData)[8 * i + 7];
          *(((char*) &val) + 1) = ((char*) ArrayData)[8 * i + 6];
          *(((char*) &val) + 2) = ((char*) ArrayData)[8 * i + 5];
          *(((char*) &val) + 3) = ((char*) ArrayData)[8 * i + 4];
          *(((char*) &val) + 4) = ((char*) ArrayData)[8 * i + 3];
          *(((char*) &val) + 5) = ((char*) ArrayData)[8 * i + 2];
          *(((char*) &val) + 6) = ((char*) ArrayData)[8 * i + 1];
          *(((char*) &val) + 7) = ((char*) ArrayData)[8 * i];

          ((double*) ArrayData)[i] = val;
        }
        break;

      default:

#if DIGFILE_DEBUG_LEVEL > 0
    printf("** Error: unknown data type\n");
#endif
        return -2; // error
      }
    }
    break;

  default:

#if DIGFILE_DEBUG_LEVEL > 0
    printf("** Error: unknown data format\n");
#endif
    return -3; // error
  }

  return 0;
}


int DIGFile::WriteArrayData(FILE* f, const void* ArrayData)
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::WriteArrayData\n");
#endif

  unsigned int i;

  switch(MainHeader.DataFormat) {
  case DIGDataFormat_ASCII:
    switch(MainHeader.DataType) {

    case DIGDataType_BOOL:
      for(i = 0; i < NoOfArrayItems; i++) {
        if(((unsigned char*) ArrayData)[i] == 0) {
          if(fprintf(f, " 0") < 0) {

#if DIGFILE_DEBUG_LEVEL > 0
            printf("** Error: writing data\n");
#endif
            return -1; // error
          }
        }
        else {
          if(fprintf(f, " 1") < 0) {

#if DIGFILE_DEBUG_LEVEL > 0
            printf("** Error: writing data\n");
#endif
            return -1; // error
          }
        }
      }
      break;

    case DIGDataType_UCHAR:
      for(i = 0; i < NoOfArrayItems; i++) {
        if(fprintf(f, " %u", ((unsigned char*) ArrayData)[i]) < 0) {

#if DIGFILE_DEBUG_LEVEL > 0
            printf("** Error: writing data\n");
#endif
          return -1; // error
        }
      }
      break;

    case DIGDataType_CHAR:
      for(i = 0; i < NoOfArrayItems; i++) {
        if(fprintf(f, " %i", ((char*) ArrayData)[i]) < 0) {

#if DIGFILE_DEBUG_LEVEL > 0
            printf("** Error: writing data\n");
#endif
          return -1; // error
        }
      }
      break;

    case DIGDataType_USHORT:
      for(i = 0; i < NoOfArrayItems; i++) {
        if(fprintf(f, " %u", ((unsigned short*) ArrayData)[i]) < 0) {

#if DIGFILE_DEBUG_LEVEL > 0
            printf("** Error: writing data\n");
#endif
          return -1; // error
        }
      }
      break;

    case DIGDataType_SHORT:
      for(i = 0; i < NoOfArrayItems; i++) {
        if(fprintf(f, " %i", ((short*) ArrayData)[i]) < 0) {

#if DIGFILE_DEBUG_LEVEL > 0
            printf("** Error: writing data\n");
#endif
          return -1; // error
        }
      }
      break;

    case DIGDataType_UINT:
      for(i = 0; i < NoOfArrayItems; i++) {
        if(fprintf(f, " %u", ((unsigned int*) ArrayData)[i]) < 0) {

#if DIGFILE_DEBUG_LEVEL > 0
            printf("** Error: writing data\n");
#endif
          return -1; // error
        }
      }
      break;

    case DIGDataType_INT:
      for(i = 0; i < NoOfArrayItems; i++) {
        if(fprintf(f, " %i", ((int*) ArrayData)[i]) < 0) {

#if DIGFILE_DEBUG_LEVEL > 0
            printf("** Error: writing data\n");
#endif
          return -1; // error
        }
      }
      break;

    case DIGDataType_FLOAT:
      for(i = 0; i < NoOfArrayItems; i++) {
        if(fprintf(f, " %f", ((double) ((float*) ArrayData)[i])) < 0) {

#if DIGFILE_DEBUG_LEVEL > 0
            printf("** Error: writing data\n");
#endif
          return -1; // error
        }
      }
      break;

    case DIGDataType_DOUBLE:
      for(i = 0; i < NoOfArrayItems; i++) {
        if(fprintf(f, " %f", ((double*) ArrayData)[i]) < 0) {

#if DIGFILE_DEBUG_LEVEL > 0
            printf("** Error: writing data\n");
#endif
          return -1; // error
        }
      }
      break;

    default:

#if DIGFILE_DEBUG_LEVEL > 0
      printf("** Error: unknown data type\n");
#endif
      return -2; // error
    }
    break;

  case DIGDataFormat_BINARY:
    // write data in curent machine endian
    if(fwrite(ArrayData, sizeof(char),ArrayBufferSize, f) != ArrayBufferSize) {
      return -1; // error
    }
    /*
    switch(MainHeader.DataType) {

    case DIGDataType_BOOL:
      if(fwrite(ArrayData, sizeof(char), NoOfArrayItems, f) != NoOfArrayItems) {
        return -1; // error
      }
      break;

    case DIGDataType_UCHAR:
      if(fwrite(ArrayData, sizeof(unsigned char), NoOfArrayItems, f) != NoOfArrayItems) {
        return -1; // error
      }
      break;

    case DIGDataType_CHAR:
      if(fwrite(ArrayData, sizeof(char), NoOfArrayItems, f) != NoOfArrayItems) {
        return -1; // error
      }
      break;

    case DIGDataType_USHORT:
      if(fwrite(ArrayData, sizeof(unsigned short), NoOfArrayItems, f) != NoOfArrayItems) {
        return -1; // error
      }
      break;

    case DIGDataType_SHORT:
      if(fwrite(ArrayData, sizeof(short), NoOfArrayItems, f) != NoOfArrayItems) {
        return -1; // error
      }
      break;

    case DIGDataType_UINT:
      if(fwrite(ArrayData, sizeof(unsigned int), NoOfArrayItems, f) != NoOfArrayItems) {
        return -1; // error
      }
      break;

    case DIGDataType_INT:
      if(fwrite(ArrayData, sizeof(int), NoOfArrayItems, f) != NoOfArrayItems) {
        return -1; // error
      }
      break;

    case DIGDataType_FLOAT:
      if(fwrite(ArrayData, sizeof(float), NoOfArrayItems, f) != NoOfArrayItems) {
        return -1; // error
      }
      break;

    case DIGDataType_DOUBLE:
      if(fwrite(ArrayData, sizeof(double), NoOfArrayItems, f) != NoOfArrayItems) {
        return -1; // error
      }
      break;

    default:
      return -2; // error
    }
    */
    break;


  default:
#if DIGFILE_DEBUG_LEVEL > 0
    printf("** Error: unknown data format\n");
#endif
    return -3; // error
  }

  return 0;
}


//////////////////////////////////////////////////////////////////////////////////////
//
// XML Parsing functions
//
//////////////////////////////////////////////////////////////////////////////////////

int DIGFile::GetUintAttributeValue(DOMElement* Element, const XMLCh* AttributeName, unsigned int* value)
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::GetUintAttributeValue\n");
#endif

  const XMLCh* AttributeValueXMLStr;
  const char* AttributeValueStr;
  AttributeValueXMLStr = Element->getAttribute(AttributeName);

  AttributeValueStr = XMLString::transcode(AttributeValueXMLStr);

  //delete AttributeValueXMLStr;

  if((*value = atoi(AttributeValueStr)) < 0) {

#if DIGFILE_DEBUG_LEVEL > 0
    printf("** Error: invalid unsigned int value\n");
#endif
    return -1;
  }

  /* // direct conversion ?

  bool XMLString::textToBin(const XMLCh *const toConvert, 
                                         unsigned int & toFill ) [static] 
  */

  return 0; // succes
}

int DIGFile::GetIntAttributeValue(DOMElement* Element, const XMLCh* AttributeName, int* value)
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::GetIntAttributeValue\n");
#endif

  const XMLCh* AttributeValueXMLStr;
  const char* AttributeValueStr;
  AttributeValueXMLStr = Element->getAttribute(AttributeName);

  AttributeValueStr = XMLString::transcode(AttributeValueXMLStr);

  //delete AttributeValueXMLStr;

  *value = atoi(AttributeValueStr);

  /* // direct conversion ?

  bool XMLString::textToBin(const XMLCh *const toConvert, 
                                         unsigned int & toFill ) [static] 
  */

  return 0; // succes
}

int DIGFile::GetDoubleAttributeValue(DOMElement* Element, const XMLCh* AttributeName, double* value)
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::GetDoubleAttributeValue\n");
#endif

  const XMLCh* AttributeValueXMLStr;
  const char* AttributeValueStr;
  AttributeValueXMLStr = Element->getAttribute(AttributeName);

  AttributeValueStr = XMLString::transcode(AttributeValueXMLStr);

  //delete AttributeValueXMLStr;

  *value = atof(AttributeValueStr);

  /* // direct conversion ?

  bool XMLString::textToBin(const XMLCh *const toConvert, 
                                         unsigned int & toFill ) [static] 
  */

  return 0; // succes
}


int DIGFile::GetTxtAttributeValue(DOMElement* Element, const XMLCh* AttributeName, char** Text)
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::GetTxtAttributeValue\n");
#endif

  const XMLCh* AttributeValueXMLStr;

  AttributeValueXMLStr = Element->getAttribute(AttributeName);

  *Text = XMLString::transcode(AttributeValueXMLStr);

  return 0; // succes
}


int DIGFile::GetElementText(DOMElement* Element, char** Text)
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::GetElementText\n");
#endif

  // get children 
  DOMNodeList* Children = Element->getChildNodes();

  // search for first instance of text node
  for(unsigned int i = 0; i < Children->getLength(); i++) {
    //const XMLCh* CXMLStr = Children->item(i)->getNodeName();
    if(Children->item(i)->getNodeType() == DOMNode::TEXT_NODE) { 
      const XMLCh* CXMLStr = Children->item(i)->getNodeValue(); 
      *Text = XMLString::transcode(CXMLStr);
      return 0;
    }
  }

#if DIGFILE_DEBUG_LEVEL > 0
    printf("** Error: Parsing - text node  not found\n");
#endif
  return -1; // text node  not found

  /*
  DOMNodeList* TextList = CommentsElement->getElementsByTagName(XMLString::transcode("text"));
  DOMNode* TextNode = TextList->item(0);
  const XMLCh* TextXMLStr = TextNode->getNodeValue();
  char* c = XMLString::transcode(TextXMLStr);
  */
}


int DIGFile::SymbolToCode(char* Symbol, const char** SymbolTable, int NumberOfSymbols)
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::SymbolToCode\n");
#endif

	for (int num = 0; num < NumberOfSymbols; num++) {
		if(strcmp(SymbolTable[num], Symbol) == 0) {
			return num;
		}  
	} 
  
#if DIGFILE_DEBUG_LEVEL > 0
    printf("** Error: Parsing - symbol not found\n");
#endif
  return -1; // symbol not found
}

int DIGFile::InitParser2(const MemBufInputSource& memBufIS)
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::InitParser2\n");
#endif

	// Initialization
	try {
		XMLPlatformUtils::Initialize();
	}  // try
	catch(const XMLException &toCatch) {
		std::cerr << "Error during Xerces-c Initialization.\n"
			<< "  Exception message:"
			<< StrX(toCatch.getMessage()) << std::endl;
		return 1;
	}  // catch

	//  Create our parser, then attach an error handler to the parser.
	//  The parser will call back to methods of the ErrorHandler if it
	//  discovers errors during the course of parsing the XML document.
	parser = new XercesDOMParser;
	parser->setValidationScheme(XercesDOMParser::Val_Auto);
	parser->setDoNamespaces(false);
	parser->setDoSchema(true);
	parser->setValidationSchemaFullChecking(true);
	parser->setCreateEntityReferenceNodes(true);
	errReporter = new DOMTreeErrorReporter();
	parser->setErrorHandler(errReporter);

	//  Parse the XML file, catching any XML exceptions that might propogate
	//  out of it.
	try {
		parser->parse(memBufIS);
	}  // try
	catch (const XMLException& e) {		
		std::cerr << "An error occurred during parsing\n   Message: "
			<< StrX(e.getMessage()) << std::endl;
		return 2;
	}  // catch
	catch (const DOMException& e) {
		std::cerr << "A DOM error occurred during parsing\n   DOMException code: "
			<< e.code << std::endl;
		return 2;
	}  // catch
	catch (...) {
		std::cerr << "An error occurred during parsing\n " << std::endl;
		return 1;
	}  // catch

	Document = ((AbstractDOMParser*)parser)->getDocument();

  // get root
	RootElement = Document->getDocumentElement();

  // get main header node
  DOMNodeList* MainHeaderList = RootElement->getElementsByTagName(XMLString::transcode("main_header"));
  MainHeaderElement = (DOMElement*) MainHeaderList->item(0);

  // get application header node
  DOMNodeList* ApplicationHeaderList = RootElement->getElementsByTagName(XMLString::transcode("application_header"));
  ApplicationHeaderElement = (DOMElement*) ApplicationHeaderList->item(0);

  // get array set header nodes
  ArraySetHeaderList = RootElement->getElementsByTagName(XMLString::transcode("array_set_header"));
  DIGFileArraySetHeader* ArraySetHeader = new DIGFileArraySetHeader();

  // select fisrt array set
  GetArraySetHeader(ArraySetHeader, 0);

  return 0;
}


int DIGFile::GetMainHeader(DIGFileMainHeader* MainHeader)
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::InitParser2\n");
#endif

  char* AttrValStr;

  // get header attributes

  // get type
  char* Type;
  if(GetTxtAttributeValue(MainHeaderElement, TypeXMLStr, &Type) != 0) {
    // error
  }

  // get title
  char* Title;
  if(GetTxtAttributeValue(MainHeaderElement, TitleXMLStr, &Title) != 0) {
    // error
  }

  // get channels
  int Channels;
  if(GetIntAttributeValue(MainHeaderElement, ChannelsXMLStr, &Channels) != 0) {
    // error
  }

  // get Number of Arrays
  int NumberOfArrays;
  if(GetIntAttributeValue(MainHeaderElement, NumberOfArraysXMLStr, &NumberOfArrays) != 0) {
    // error
  }

  // get Number of Array Sets
  int NumberOfSets;
  if(GetIntAttributeValue(MainHeaderElement, NumberOfArraySetsXMLStr, &NumberOfSets) != 0) {
    // error
  }

  // get Endian
  if(GetTxtAttributeValue(MainHeaderElement, EndianXMLStr, &AttrValStr) != 0) {
    // error
  }
  
  if((Endian = (DIGEndian) SymbolToCode(AttrValStr, DIGEndianStr, DIGEndianSize)) < 0) {
    // error
  }
  
  // get Value Type
  if(GetTxtAttributeValue(MainHeaderElement, ValueTypeXMLStr, &AttrValStr) != 0) {
    // error
  }


  DIGValueType ValueType;
  if((ValueType = (DIGValueType) SymbolToCode(AttrValStr, DIGValueTypeStr, DIGValueTypeSize)) < 0) {
    // error
  }

  // get Data Type
  if(GetTxtAttributeValue(MainHeaderElement, DataTypeXMLStr, &AttrValStr) != 0) {
    // error
  }

  DIGDataType DataType;
  if((DataType = (DIGDataType) SymbolToCode(AttrValStr, DIGDataTypeStr, DIGDataTypeSize)) < 0) {
    // error
  }

  // get Data Format
  if(GetTxtAttributeValue(MainHeaderElement, DataFormatXMLStr, &AttrValStr) != 0) {
    // error
  }

  DIGDataFormat DataFormat;
  if((DataFormat = (DIGDataFormat) SymbolToCode(AttrValStr, DIGDataFormatStr, DIGDataFormatSize)) < 0) {
    // error
  }

  // get Grid Type
  if(GetTxtAttributeValue(MainHeaderElement, GridTypeXMLStr, &AttrValStr) != 0) {
    // error
  }

  DIGGrid Grid;
  if((Grid = (DIGGrid) SymbolToCode(AttrValStr, DIGGridStr, DIGGridSize)) < 0) {
    // error
  }

  // get Basis Function
  if(GetTxtAttributeValue(MainHeaderElement, BasisFunctionXMLStr, &AttrValStr) != 0) {
    // error
  }

  DIGBasis Basis;
  if((Basis = (DIGBasis) SymbolToCode(AttrValStr, DIGBasisStr, DIGBasisSize)) < 0) {
    // error
  }

  // get Unit
  if(GetTxtAttributeValue(MainHeaderElement, UnitXMLStr, &AttrValStr) != 0) {
    // error
  }

  DIGUnit Unit;
  if((Unit = (DIGUnit) SymbolToCode(AttrValStr, DIGUnitStr, DIGUnitSize)) < 0) {
    // error
  }

  // get Other Unit
  char* OtherUnit;
  if(GetTxtAttributeValue(MainHeaderElement, OtherUnitXMLStr, &OtherUnit) != 0) {
    // error
  }

  // get other header values
  // get comments
  // get Comment Element

  char* Comment = DIGEmptyStr;

  DOMNodeList* CommentsList = MainHeaderElement->getElementsByTagName(CommentsXMLStr);

  int tNumC = CommentsList->getLength();
  if(tNumC > 0)	{

    DOMElement* CommentsElement = (DOMElement*) CommentsList->item(0);

    // get Comment Text
    if(GetElementText(CommentsElement, &Comment) != 0) {
      ;// allow empty comments
    }
  }

  DIGDimensions Dimensions;

  DOMNodeList* DimensionsList = MainHeaderElement->getElementsByTagName(DimensionsXMLStr);
  DOMElement* DimensionsElement = (DOMElement*) DimensionsList->item(0);

  // get Dimmension X
  if(GetIntAttributeValue(DimensionsElement, DimmensionXXMLStr, &(Dimensions.x)) != 0) {
    // error
  }

  // get Dimmension Y
  if(GetIntAttributeValue(DimensionsElement, DimmensionYXMLStr, &(Dimensions.y)) != 0) {
    // error
  }

  // get Dimmension Z
  if(GetIntAttributeValue(DimensionsElement, DimmensionZXMLStr, &(Dimensions.z)) != 0) {
    // error
  }

  // get Sampling Rate X
  DIGSampling SamplingX;

  DOMNodeList* SamplingXList = MainHeaderElement->getElementsByTagName(SamplingRateXXMLStr);
  DOMElement* SamplingXElement = (DOMElement*) SamplingXList->item(0);


  if(GetDoubleAttributeValue(SamplingXElement, DimmensionXXMLStr, &(SamplingX.x)) != 0) {
    // error
  }

  if(GetDoubleAttributeValue(SamplingXElement, DimmensionYXMLStr, &(SamplingX.y)) != 0) {
    // error
  }

  if(GetDoubleAttributeValue(SamplingXElement, DimmensionZXMLStr, &(SamplingX.z)) != 0) {
    // error
  }


  // get Sampling Rate Y
  DIGSampling SamplingY;

  DOMNodeList* SamplingYList = MainHeaderElement->getElementsByTagName(SamplingRateYXMLStr);
  DOMElement* SamplingYElement = (DOMElement*) SamplingYList->item(0);


  if(GetDoubleAttributeValue(SamplingYElement, DimmensionXXMLStr, &(SamplingY.x)) != 0) {
    // error
  }

  if(GetDoubleAttributeValue(SamplingYElement, DimmensionYXMLStr, &(SamplingY.y)) != 0) {
    // error
  }

  if(GetDoubleAttributeValue(SamplingYElement, DimmensionZXMLStr, &(SamplingY.z)) != 0) {
    // error
  }

  // get Sampling Rate Z
  DIGSampling SamplingZ;

  DOMNodeList* SamplingZList = MainHeaderElement->getElementsByTagName(SamplingRateZXMLStr);
  DOMElement* SamplingZElement = (DOMElement*) SamplingZList->item(0);


  if(GetDoubleAttributeValue(SamplingZElement, DimmensionXXMLStr, &(SamplingZ.x)) != 0) {
    // error
  }

  if(GetDoubleAttributeValue(SamplingZElement, DimmensionYXMLStr, &(SamplingZ.y)) != 0) {
    // error
  }

  if(GetDoubleAttributeValue(SamplingZElement, DimmensionZXMLStr, &(SamplingZ.z)) != 0) {
    // error
  }

  NoOfArrays = NumberOfArrays;
  NoOfSets = NumberOfSets;

  MainHeader->Title.Set(Title);
  MainHeader->Type.Set(Type);
  MainHeader->Channels = Channels;
  //MainHeader->Endian = Endian;
  MainHeader->ValueType = ValueType;
  MainHeader->DataType = DataType;
  MainHeader->DataFormat = DataFormat;
  MainHeader->Grid = Grid;
  MainHeader->Basis = Basis;
  MainHeader->Unit = Unit;
  MainHeader->OtherUnit.Set(OtherUnit);
  MainHeader->Comment.Set(Comment);
  MainHeader->Dimensions = Dimensions;
  MainHeader->SamplingX = SamplingX;
  MainHeader->SamplingY = SamplingY;
  MainHeader->SamplingZ = SamplingZ;
  return 0;
}


int DIGFile::ParseApplicationHeader() {
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::ParseApplicationHeader\n");
#endif
  return 0;
}


int DIGFile::GetArraySetHeader(DIGFileArraySetHeader* ArraySetHeader, unsigned int index)
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::GetArraySetHeader\n");
#endif

  if(index > ArraySetHeaderList->getLength()) {

#if DIGFILE_DEBUG_LEVEL > 0
    printf("** Error: index out of range\n");
#endif
    // error index out of range
  }

  DOMElement* ArraySetHeaderElement = (DOMElement*) ArraySetHeaderList->item(index);

  // get Array Set Header Attributes

  // get type
  char* Type;
  if(GetTxtAttributeValue(ArraySetHeaderElement, TypeXMLStr, &Type) != 0) {
    // error
  }

  // get title
  char* Title;
  if(GetTxtAttributeValue(ArraySetHeaderElement, TitleXMLStr, &Title) != 0) {
    // error
  }

  // get Comment Element
  char* Comment = DIGEmptyStr;

  DOMNodeList* CommentsList = ArraySetHeaderElement->getElementsByTagName(CommentsXMLStr);

  int tNumC = CommentsList->getLength();
  if(tNumC > 0)	{

    DOMElement* CommentsElement = (DOMElement*) CommentsList->item(0);

    // get Comment Text
    if(GetElementText(CommentsElement, &Comment) != 0) {
      ;// allow empty comments
    }
  }

  // get Parameters Element
  DOMNodeList* ParametersList = ArraySetHeaderElement->getElementsByTagName(ParametersXMLStr);
  DOMElement* ParametersElement = (DOMElement*) ParametersList->item(0);

  // get Parameters Text
  char* Parameters;
  if(GetElementText(ParametersElement, &Parameters) != 0) {
    // error
    Parameters = DIGEmptyStr;
  }

  ArraySetHeader->Type.Set(Type);
  ArraySetHeader->Title.Set(Title);
  ArraySetHeader->Comment.Set(Comment);
  ArraySetHeader->Parameters.Set(Parameters);

  // load array list
  ArrayHeaderList = ArraySetHeaderElement->getElementsByTagName(ArrayHeaderXMLStr);

  return 0;
}


int DIGFile::GetArrayHeader(DIGFileArrayHeader* ArrayHeader, unsigned int index)
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::GetArrayHeader\n");
#endif

  if(index > ArrayHeaderList->getLength()) {
    // error index out of range
  }

  DOMElement* ArrayHeaderElement = (DOMElement*) ArrayHeaderList->item(index);

  // get Array Header Attributes

  // get Enumeration Number
  int EnumNo;

  if(GetIntAttributeValue(ArrayHeaderElement, EnumerationNumberXMLStr, &EnumNo) != 0) {
    return -1; // error
  }

  // get Comment Element
  char* Comment = DIGEmptyStr;

  DOMNodeList* CommentsList = ArrayHeaderElement->getElementsByTagName(CommentsXMLStr);

  int tNumC = CommentsList->getLength();
  if(tNumC > 0)	{

    DOMElement* CommentsElement = (DOMElement*) CommentsList->item(0);

    // get Comment Text
    if(GetElementText(CommentsElement, &Comment) != 0) {
      ;// allow empty comments
    }
  }

  ArrayHeader->EnumNo = EnumNo;
  ArrayHeader->Comment.Set(Comment);

  return 0;
}


int DIGFile::ResetParser()
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::ResetParser\n");
#endif

	//  Clean up the error handler. The parser does not adopt handlers
	//  since they could be many objects or one object installed for multiple
	//  handlers.
	delete errReporter;

	//  Delete the parser itself.  Must be done prior to calling Terminate, below.
	delete parser;

	// And call the termination method
	XMLPlatformUtils::Terminate();

	return 0; 
}


int DIGFile::ParseXML(const char* MemBuf)
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::ParseXML\n");
#endif

  static const char*  gMemBufId = "XMLHeader";

  MemBufInputSource* memBufIS = new MemBufInputSource
  (
    (const XMLByte*)MemBuf,
    strlen(MemBuf),
    gMemBufId,
    false
  );

  // initialize parser and pharse root
  if(InitParser2(*memBufIS) != 0) {

#if DIGFILE_DEBUG_LEVEL > 0
    printf("** Error: initializing parser and gtting doument root\n");
#endif
    return -1; // error
  }

  // parse main header
  if(GetMainHeader(&MainHeader)) {

#if DIGFILE_DEBUG_LEVEL > 0
    printf("** Error: parsing main header\n");
#endif
    return -1; // error
  }

  // parse application header
  if(ParseApplicationHeader()) {

#if DIGFILE_DEBUG_LEVEL > 0
    printf("** Error: parsing application header\n");
#endif
    return -1; // error
  }

  // parse arrayset headers
  unsigned int NumberOfArrays = 0;

  for(unsigned int i = 0; i < ArraySetHeaderList->getLength(); i++) {

    // create new array set
    ArraySet* ASet = new ArraySet;
    ASet->FirstArrayOffset = NumberOfArrays;

    // create new array set header
    ASet->ArraySetHeader = new DIGFileArraySetHeader;

    // load it with data
    if(GetArraySetHeader(ASet->ArraySetHeader, i) != 0) {

#if DIGFILE_DEBUG_LEVEL > 0
    printf("** Error: parsing array set header\n");
#endif
      return -1; // error
    }

    // append arrays
    for(unsigned int j = 0; j < ArrayHeaderList->getLength(); j++) {

      // create new array header
      DIGFileArrayHeader* ArrayHeader = new DIGFileArrayHeader;

      // load it with data
      if(GetArrayHeader(ArrayHeader, j) != 0) {

#if DIGFILE_DEBUG_LEVEL > 0
    printf("** Error: parsing array header\n");
#endif
        return -1; //error
      }

      // append to array list
      ASet->Append(ArrayHeader);

      NumberOfArrays++;
    }

    // append array set to array set list
    ArraySets.Append(ASet);
  }

  if(NumberOfArrays != NoOfArrays) {

#if DIGFILE_DEBUG_LEVEL > 0
    printf("** Error: invalid number of arrays\n");
#endif
    return -1; // error
  }

  ResetParser();
   
  return 0;
}

//////////////////////////////////////////////////////////////////////////////////////
//
// XML writing functions
//
//////////////////////////////////////////////////////////////////////////////////////

int DIGFile::WriteXMLHeader()
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::WriteXMLHeader\n");
#endif

  // create XML buffer
  XMLFormatTarget* mymFormTarget = new MemBufFormatTarget(4*1023);


  // create DOMDocument
  CreateRootElement();

  if(CreateMainHeader(&MainHeader) != 0) {

#if DIGFILE_DEBUG_LEVEL > 0
    printf("** Error: creating main header\n");
#endif
    return -1;
  }

  if(CreateApplicationHeader() != 0) {

#if DIGFILE_DEBUG_LEVEL > 0
    printf("** Error: creating application header\n");
#endif
    return -2;
  }

  // step through array sets
  for(unsigned int i = 0; i < ArraySets.getLength(); i++) {

    ArraySet* ASet;

    // get array set from set list
    if(ArraySets.getArraySet(&ASet, i) != 0) {

#if DIGFILE_DEBUG_LEVEL > 0
    printf("** Error: creating array set header\n");
#endif
      return -3; // error
    }

    // write array set header
    AppendArraySetHeader(ASet->ArraySetHeader); 

    // step through arrays
    for(unsigned int j = 0; j < ASet->getLength(); j++) {

      DIGFileArrayHeader* ArrayHeader;

      // get array header
      ASet->getArrayHeader(&ArrayHeader, j);

      // write array header
      AppendArrayHeader(ArrayHeader);
    }
  }

  //WriteSufix();

  // serialize 
  ///DOMPrintFilter   *myFilter = 0;

  try {
		// get a serializer, an instance of DOMWriter (kmr Now DOMLSSerializer)
		XMLCh tempStr[100];
		XMLString::transcode("LS", tempStr, 99);
		DOMImplementation *impl          = DOMImplementationRegistry::getDOMImplementation(tempStr);
		DOMLSSerializer   *theSerializer = ((DOMImplementationLS*)impl)->createLSSerializer();
                DOMConfiguration  *dc            = theSerializer->getDomConfig();
                DOMLSOutput* theOutput = ((DOMImplementationLS*)impl)->createLSOutput();
                theOutput->setByteStream(mymFormTarget);
                
		// set user specified end of line sequence and output encoding
		//theSerializer->setNewLine(gMyEOLSequence);

    theSerializer->setNewLine(XMLString::transcode("\x0A"));

    static XMLCh* gOutputEncoding = 0;

    // kmr theSerializer->setEncoding(gOutputEncoding);
    theOutput->setEncoding(gOutputEncoding);
    
		// plug in user's own filter
    /*
		if(gUseFilter) {
			// even we say to show attribute, but the DOMWriter
			// will not show attribute nodes to the filter as
			// the specs explicitly says that DOMWriter shall
			// NOT show attributes to DOMWriterFilter.
			//
			// so DOMNodeFilter::SHOW_ATTRIBUTE has no effect.
			// same DOMNodeFilter::SHOW_DOCUMENT_TYPE, no effect.
			//
			myFilter = new DOMPrintFilter(DOMNodeFilter::SHOW_ELEMENT   |
                                            DOMNodeFilter::SHOW_ATTRIBUTE |
                                            DOMNodeFilter::SHOW_DOCUMENT_TYPE
											);
			theSerializer->setFilter(myFilter);
		}
    */

		// plug in user's own error handler
		DOMErrorHandler* myErrorHandler = new DOMPrintErrorHandler();
		dc->setParameter(XMLUni::fgDOMErrorHandler,myErrorHandler);

		// set feature if the serializer supports the feature/mode
    
    static bool gSplitCdataSections = true;

		if(dc->canSetParameter(XMLUni::fgDOMWRTSplitCdataSections, gSplitCdataSections))
			dc->setParameter(XMLUni::fgDOMWRTSplitCdataSections, gSplitCdataSections);

    static bool gDiscardDefaultContent = true;
		
		if(dc->canSetParameter(XMLUni::fgDOMWRTDiscardDefaultContent, gDiscardDefaultContent))
                        dc->setParameter(XMLUni::fgDOMWRTDiscardDefaultContent, gDiscardDefaultContent);

    static bool gFormatPrettyPrint = true;

		if(dc->canSetParameter(XMLUni::fgDOMWRTFormatPrettyPrint, gFormatPrettyPrint))
			dc->setParameter(XMLUni::fgDOMWRTFormatPrettyPrint, gFormatPrettyPrint);

    //
		// Plug in a format target to receive the resultant
		// XML stream from the serializer.
          //
		// StdOutFormatTarget prints the resultant XML stream
		// to stdout once it receives any thing from the serializer.
		//
		
    //XMLFormatTarget* myFormTarget = new StdOutFormatTarget();
		//theSerializer->writeNode(myFormTarget, *Document);

		// get the DOM representation
		//DOMNode                     *doc = parser->getDocument();

		//
		// do the serialization through DOMWriter::writeNode();
		//
		//theSerializer->writeNode(myFormTarget, *Document);

		//XMLFormatTarget* myfFormTarget = new LocalFileFormatTarget("xxx");
		//theSerializer->writeNode(myfFormTarget, *Document);

    //((LocalFileFormatTarget*) myfFormTarget)->Target.setEscapeFlags();

                //kmr		theSerializer->writeNode(mymFormTarget, *Document);
                theSerializer->write(Document, theOutput);
                delete theOutput;
		delete theSerializer;

    //
		// Filter, formatTarget and error handler
		// are NOT owned by the serializer.
		//

		delete myErrorHandler;

    /*
		if (gUseFilter)
			delete myFilter; 
    */

  }
  catch(XMLException& e) {
    std::cerr << "An error occurred during creation of output transcoder. Msg is:"
        << std::endl
        << StrX(e.getMessage()) << std::endl;
    //retval = 4;
  }

  const XMLByte* XMLbuf = ((MemBufFormatTarget*) mymFormTarget)->getRawBuffer();

  fprintf(File, "%s", XMLbuf);

  delete mymFormTarget;

  return 0;
}


// Create root element
//void DIGFile::WritePrefix()
void DIGFile::CreateRootElement()
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::CreateRootElement\n");
#endif

  Document = impl->createDocument(
    0,            // root element namespace URI.
    RootXMLStr,   // root element name
    0             // document type object (DTD).
  );                  

  RootElement = Document->getDocumentElement();

/*
	fprintf(File, "<?xml version=\"1.0\"?>\n\n");
	fprintf(File, "<DIG_DATA\n");
	fprintf(File, "	xmlns:xsd=\"http://www.w3.org/2001/XMLSchema-instance\"\n");
	fprintf(File, "	xsd:noNamespcadSchemaLocation=\"%s DIGschema.xsd\"\n", SchemaName);
	fprintf (File, ">\n");
  */
}


// Create Main Header
int DIGFile::CreateMainHeader(DIGFileMainHeader* header)
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::CreateMainHeader\n");
#endif

  XMLCh* tmpXMLStr;

  // create main header node
  MainHeaderElement = Document->createElement(MainHeaderXMLStr);

  // attributes

  // define type
  tmpXMLStr = XMLString::transcode(header->Type.Get());
  MainHeaderElement->setAttribute(TypeXMLStr, tmpXMLStr);
  //delete tmpXMLStr;

  // define title
  tmpXMLStr = XMLString::transcode(header->Title.Get());
  MainHeaderElement->setAttribute(TitleXMLStr, tmpXMLStr);
  //delete tmpXMLStr;

  // define channels
  //tmpXMLStr = new XMLCh[5];
  //XMLString::binToText(header->Channels, tmpXMLStr, 4, 10);
  unsigned int maxchars = 1;
  if (header->Channels != 0)
    maxchars += static_cast<unsigned int>(log10(static_cast<double>(header->Channels)));
  tmpXMLStr = new XMLCh[maxchars+1];
  XMLString::binToText(header->Channels, tmpXMLStr, maxchars, 10);
  MainHeaderElement->setAttribute(ChannelsXMLStr, tmpXMLStr);
  delete tmpXMLStr;

  // define number of arrays
  //tmpXMLStr = new XMLCh[5];
  //XMLString::binToText(NoOfArrays, tmpXMLStr, 4, 10);
  maxchars = 1;
  if (NoOfArrays != 0)
    maxchars += static_cast<unsigned int>(log10(static_cast<double>(NoOfArrays)));
  tmpXMLStr = new XMLCh[maxchars+1];
  XMLString::binToText(NoOfArrays, tmpXMLStr, maxchars, 10);
  MainHeaderElement->setAttribute(NumberOfArraysXMLStr, tmpXMLStr);
  delete tmpXMLStr;

  // define number of array sets
  //tmpXMLStr = new XMLCh[5];
  //XMLString::binToText(NoOfSets, tmpXMLStr, 4, 10);
  maxchars = 1;
  if (NoOfSets != 0)
    maxchars += static_cast<unsigned int>(log10(static_cast<double>(NoOfSets)));
  tmpXMLStr = new XMLCh[maxchars+1];
  XMLString::binToText(NoOfSets, tmpXMLStr, maxchars, 10);
  MainHeaderElement->setAttribute(NumberOfArraySetsXMLStr, tmpXMLStr);
  delete tmpXMLStr;

  // define endian according to curent machine type
  tmpXMLStr = XMLString::transcode(DIGEndianStr[static_cast<int>(DIG_ENDIAN)]);
  MainHeaderElement->setAttribute(EndianXMLStr, tmpXMLStr);
  //delete tmpXMLStr;

  // define value type
  tmpXMLStr = XMLString::transcode(DIGValueTypeStr[header->ValueType]);
  MainHeaderElement->setAttribute(ValueTypeXMLStr, tmpXMLStr);
  //delete tmpXMLStr;

  // define data type
  tmpXMLStr = XMLString::transcode(DIGDataTypeStr[header->DataType]);
  MainHeaderElement->setAttribute(DataTypeXMLStr, tmpXMLStr);
  //delete tmpXMLStr;

  // define data format
  tmpXMLStr = XMLString::transcode(DIGDataFormatStr[header->DataFormat]);
  MainHeaderElement->setAttribute(DataFormatXMLStr, tmpXMLStr);
  //delete tmpXMLStr;

  // define grid type
  tmpXMLStr = XMLString::transcode(DIGGridStr[header->Grid]);
  MainHeaderElement->setAttribute(GridTypeXMLStr, tmpXMLStr);
  //delete tmpXMLStr;

  // define basis function
  tmpXMLStr = XMLString::transcode(DIGBasisStr[header->Basis]);
  MainHeaderElement->setAttribute(BasisFunctionXMLStr, tmpXMLStr);
  //delete tmpXMLStr;

  // define unit
  tmpXMLStr = XMLString::transcode(DIGUnitStr[header->Unit]);
  MainHeaderElement->setAttribute(UnitXMLStr, tmpXMLStr);
  //delete tmpXMLStr;

  // define other unit
	if(header->OtherUnit.Get() != NULL) {
    MainHeaderElement->setAttribute(OtherUnitXMLStr, XMLString::transcode(header->OtherUnit.Get()));
	}

  // children


  // define comments
	if(header->Comment.Get() != NULL) {
    
    // create comments node
    DOMElement* Comments = Document->createElement(CommentsXMLStr);

    // create comments text node
    tmpXMLStr = XMLString::transcode(header->Comment.Get());
    DOMText* CommentText = Document->createTextNode(tmpXMLStr);
    //delete tmpXMLStr;
    Comments->appendChild(CommentText);

    // append comments to main header
    MainHeaderElement->appendChild(Comments);
  }


  // create dimensions node
  DOMElement* Dimensions = Document->createElement(DimensionsXMLStr);

	const DIGDimensions* dim = &(header->Dimensions);

  // set dimension attributes
  //tmpXMLStr = new XMLCh[5];
  //XMLString::binToText(dim->x, tmpXMLStr, 4, 10);
  maxchars = 1;
  if(dim->x != 0) {
    maxchars += static_cast<unsigned int>(log10(static_cast<double>(dim->x)));
  }
  tmpXMLStr = new XMLCh[maxchars+1];
  XMLString::binToText(dim->x, tmpXMLStr, maxchars, 10);
  Dimensions->setAttribute(DimmensionXXMLStr, tmpXMLStr);
  delete tmpXMLStr;

  //XMLString::binToText(dim->y, tmpXMLStr, 4, 10);
  maxchars = 1;
  if(dim->y != 0) {
    maxchars += static_cast<unsigned int>(log10(static_cast<double>(dim->y)));
  }
  tmpXMLStr = new XMLCh[maxchars+1];
  XMLString::binToText(dim->y, tmpXMLStr, maxchars, 10);
  Dimensions->setAttribute(DimmensionYXMLStr, tmpXMLStr);
  delete tmpXMLStr;

  //XMLString::binToText(dim->z, tmpXMLStr, 4, 10);
  maxchars = 1;
  if(dim->z != 0) {
    maxchars += static_cast<unsigned int>(log10(static_cast<double>(dim->z)));
  }
  tmpXMLStr = new XMLCh[maxchars+1];
  XMLString::binToText(dim->z, tmpXMLStr, maxchars, 10);
  Dimensions->setAttribute(DimmensionZXMLStr, tmpXMLStr);
  delete tmpXMLStr;

  // append dimensions to main header
  MainHeaderElement->appendChild(Dimensions);


  // create sampling rate X node
  DOMElement* SamplingRateX = Document->createElement(SamplingRateXXMLStr);

	DIGSampling* sampX = &(header->SamplingX);

  // set sampling rate X attributes
  char valStr[20];

  sprintf(valStr, "%f", sampX->x);
  tmpXMLStr = XMLString::transcode(valStr);
  SamplingRateX->setAttribute(DimmensionXXMLStr, tmpXMLStr);
  //delete tmpXMLStr;

  sprintf(valStr, "%f", sampX->y);
  tmpXMLStr = XMLString::transcode(valStr);
  SamplingRateX->setAttribute(DimmensionYXMLStr, tmpXMLStr);
  //delete tmpXMLStr;

  sprintf(valStr, "%f", sampX->z);
  tmpXMLStr = XMLString::transcode(valStr);
  SamplingRateX->setAttribute(DimmensionZXMLStr, tmpXMLStr);
  //delete tmpXMLStr;

  // append sampling rate X to main header
  MainHeaderElement->appendChild(SamplingRateX);


  // create sampling rate Y node
  DOMElement* SamplingRateY = Document->createElement(SamplingRateYXMLStr);

	DIGSampling* sampY = &(header->SamplingY);

  // set sampling rate Y attributes
  sprintf(valStr, "%f", sampY->x);
  tmpXMLStr = XMLString::transcode(valStr);
  SamplingRateY->setAttribute(DimmensionXXMLStr, tmpXMLStr);
  //delete tmpXMLStr;

  sprintf(valStr, "%f", sampY->y);
  tmpXMLStr = XMLString::transcode(valStr);
  SamplingRateY->setAttribute(DimmensionYXMLStr, tmpXMLStr);
  //delete tmpXMLStr;

  sprintf(valStr, "%f", sampY->z);
  tmpXMLStr = XMLString::transcode(valStr);
  SamplingRateY->setAttribute(DimmensionZXMLStr, tmpXMLStr);
  //delete tmpXMLStr;

  // append sampling rate Y to main header
  MainHeaderElement->appendChild(SamplingRateY);


  // create sampling rate Z node
  DOMElement* SamplingRateZ = Document->createElement(SamplingRateZXMLStr);

	DIGSampling* sampZ = &(header->SamplingZ);

  // set sampling rate Z attributes
  sprintf(valStr, "%f", sampZ->x);
  tmpXMLStr = XMLString::transcode(valStr);
  SamplingRateZ->setAttribute(DimmensionXXMLStr, tmpXMLStr);
  //delete tmpXMLStr;

  sprintf(valStr, "%f", sampZ->y);
  tmpXMLStr = XMLString::transcode(valStr);
  SamplingRateZ->setAttribute(DimmensionYXMLStr, tmpXMLStr);
  //delete tmpXMLStr;

  sprintf(valStr, "%f", sampZ->z);
  tmpXMLStr = XMLString::transcode(valStr);
  SamplingRateZ->setAttribute(DimmensionZXMLStr, tmpXMLStr);
  //delete tmpXMLStr;

  // append sampling rate Z to main header
  MainHeaderElement->appendChild(SamplingRateZ);


  // append main header to document root
  RootElement->appendChild(MainHeaderElement);

  /*
  if(MainHeaderWritten) {
    return -1; // main header already in file
  }

	fprintf(File, "  <main_header\n");
 	fprintf(File, "    type=\"%s\"\n", header->GetType());
 	fprintf(File, "    title=\"%s\"\n", header->GetTitle());
	fprintf(File, "    channels=\"%d\"\n", header->GetChannels());
	fprintf(File, "    number_of_arrays=\"%d\"\n", header->GetNumberOfArrays());
	fprintf(File, "    number_of_array_sets=\"%d\"\n", header->GetNumberOfSets());
	fprintf(File, "    endian=\"%s\"\n", header->GetEndianStr());
	fprintf(File, "    value_type=\"%s\"\n", header->GetValueTypeStr());
	fprintf(File, "    data_type=\"%s\"\n", header->GetDataTypeStr());
	fprintf(File, "    data_format=\"%s\"\n", header->GetDataFormatStr());
	fprintf(File, "    grid_type=\"%s\"\n", header->GetGridStr());
	fprintf(File, "    basis_function=\"%s\"\n", header->GetBasisStr());
	fprintf(File, "    unit=\"%s\"\n", header->GetUnitStr());


	if(header->GetOtherUnit() != NULL) {
		fprintf(File, "    other_unit=\"%s\"\n", header->GetOtherUnit());	
	}

	fprintf(File, "  >\n");	

	if(header->GetComment() != NULL){
		fprintf (File, "    <comments>\n");
		fprintf (File, "    %s\n", header->GetComment());
		fprintf (File, "    </comments>\n");
  }

	DIGDimensions *dim = header->GetDimensions();
	fprintf(File, "    <dimensions x=\"%d\" y=\"%d\" z=\"%d\"/>\n", dim->x, dim->y, dim->z);

	DIGSampling *sampX = header->GetSamplingX();
	fprintf(File, "    <sampling_rate_x x=\"%6.3f\" y=\"%6.3f\" z=\"%6.3f\"/>\n", sampX->x, sampX->y, sampX->z);

	DIGSampling *sampY = header->GetSamplingY();
	fprintf(File, "    <sampling_rate_y x=\"%6.3f\" y=\"%6.3f\" z=\"%6.3f\"/>\n", sampY->x, sampY->y, sampY->z);

	DIGSampling *sampZ = header->GetSamplingZ();
	fprintf(File, "    <sampling_rate_z x=\"%6.3f\" y=\"%6.3f\" z=\"%6.3f\"/>\n", sampZ->x, sampZ->y, sampZ->z);

	fprintf(File, "  </main_header>\n\n");
	//fprintf(File, "  <!-- origin is %d %d %d -->\n", (dim->x-1)/2, (dim->y-1)/2, (dim->z-1)/2);
	

  MainHeaderWritten = true;
  */
	return 0;
}

int DIGFile::CreateApplicationHeader()
{

#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::CreateApplicationHeader\n");
#endif

	return 0;
}



int DIGFile::AppendArraySetHeader(DIGFileArraySetHeader* SetHeader)
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::AppendArraySetHeader\n");
#endif

  XMLCh* tmpXMLStr;

  // create array set header node
  ArraySetHeaderElement = Document->createElement(ArraySetHeaderXMLStr);

  // attributes

  // define type
  tmpXMLStr = XMLString::transcode(SetHeader->Type.Get());
  ArraySetHeaderElement->setAttribute(TypeXMLStr, tmpXMLStr);
  //delete tmpXMLStr;

  // define title
  tmpXMLStr = XMLString::transcode(SetHeader->Title.Get());
  ArraySetHeaderElement->setAttribute(TitleXMLStr, tmpXMLStr);
  //delete tmpXMLStr;


  // children


  // define comments
  const char* Com = SetHeader->Comment.Get();

  if(Com != NULL) {
    
    // create comments node
    DOMElement* Comments = Document->createElement(CommentsXMLStr);

    // create comments text node
    tmpXMLStr = XMLString::transcode(Com);
    DOMText* CommentText = Document->createTextNode(tmpXMLStr);
    //delete tmpXMLStr;
    Comments->appendChild(CommentText);

    // append comments to array set header
    ArraySetHeaderElement->appendChild(Comments);
  }

  // create parameters node
  DOMElement* Parameter = Document->createElement(ParametersXMLStr);

  // create parameters text node
  tmpXMLStr = XMLString::transcode(SetHeader->Parameters.Get());
  DOMText* ParameterText = Document->createTextNode(tmpXMLStr);
  //delete tmpXMLStr;
  Parameter->appendChild(ParameterText);

  // append parameters to array set header
  ArraySetHeaderElement->appendChild(Parameter);


  // append array set header to main header
  RootElement->appendChild(ArraySetHeaderElement);

  /*
  // check if main header written
  if(!MainHeaderWritten) {
    return -1; // error: no main header in file
  }

  // close previous array set
  if(ArraySetOpen) {
    fprintf(File, "  </array_set_header>\n\n");
  }
  else {
    ArraySetOpen = true;
  }
//  ArrayCounter = 0;

	fprintf(File, "\n  <array_set_header\n");
	fprintf(File, "    type=\"%s\"\n", SetHeader->GetType());
	fprintf(File, "    title=\"%s\"\n", SetHeader->GetTitle());
	fprintf(File, "  >\n");

  char* Comments = SetHeader->GetComments();

  if(Comments != NULL) {
		fprintf(File, "    <comments>\n");
		fprintf(File, "      %s\n", Comments);
		fprintf(File, "    </comments>\n");
	}
  
	fprintf(File, "    <parameters>\n");
	fprintf(File, "      %s\n", SetHeader->GetParameters());
	fprintf(File, "    </parameters>\n");
*/

	return 0;
}



int DIGFile::AppendArrayHeader(DIGFileArrayHeader* ArrayHeader)
{
#if DIGFILE_DEBUG_LEVEL > 2
  fprintf(stderr, "DIGFile::AppendArrayHeader\n");
#endif

  XMLCh* tmpXMLStr;

  // create array set header node
  ArrayHeaderElement = Document->createElement(ArrayHeaderXMLStr);

  // attributes

  // define type
  //tmpXMLStr = new XMLCh[5];
  //XMLString::binToText(ArrayHeader->EnumNo, tmpXMLStr, 4, 10);
  unsigned int maxchars = 1;
  if(ArrayHeader->EnumNo != 0) {
    maxchars += static_cast<unsigned int>(log10(static_cast<double>(ArrayHeader->EnumNo)));
  }
  tmpXMLStr = new XMLCh[maxchars+1];
  XMLString::binToText(ArrayHeader->EnumNo, tmpXMLStr, maxchars, 10);
  ArrayHeaderElement->setAttribute(EnumerationNumberXMLStr, tmpXMLStr);
  delete tmpXMLStr;


  // children


  // define comments
  const char* Com = ArrayHeader->Comment.Get();

  if(Com != NULL) {
    
    // create comments node
    DOMElement* Comments = Document->createElement(CommentsXMLStr);

    // create comments text node
    tmpXMLStr = XMLString::transcode(Com);
    DOMText* CommentText = Document->createTextNode(tmpXMLStr);
    //delete tmpXMLStr;
    Comments->appendChild(CommentText);

    // append comments to array header
    ArrayHeaderElement->appendChild(Comments);
  }


  // append array header to active array set header
  ArraySetHeaderElement->appendChild(ArrayHeaderElement);

  /*
  fprintf(File, "\n    <array_header\n");

	//fprintf(File, "      enumeration_number=\"%d\">\n", array_enumeration[sets*size+arrays]);
  //fprintf(File, "      enumeration_number=\"%d\"\n", ArrayCounter);
  fprintf(File, "      enumeration_number=\"%d\"\n", ArrayHeader->GetEnumeration());
  fprintf(File, "    >\n");

  
  char* comment = ArrayHeader->GetComment();

  if(comment != NULL) {
		fprintf(File, "      <comments>\n");
		fprintf(File, "        %s\n", comment);
    fprintf(File, "      </comments>\n");
	}

	fprintf(File, "    </array_header>\n");
*/
	return 0;
}
