/* 
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/emap_CInitMAP.cpp $
 $LastChangedRevision: 85 $
 $Date: 2014-07-02 16:07:08 -0400 (Wed, 02 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 ===========================================================================
 initialize working space for the MAP algorithm
 ===========================================================================
 */

#include <cstdio>

#include "blkdta.h"
#include "geom.h"
#include "uiod.h"
#include "anglst.h"
#include "wray.h"
#include "blob.h"

#include "emap.h"

REAL getSjj(INTEGER j, INTEGER nelem);

void emap_class::InitMAP(REAL* recon)
{

	INTEGER testneg;
	//------ local variables
	INTEGER i, first, np, nr, idelta, numb, list[4096], usnr;
	REAL snorm, weight[4096];

	INTEGER area;

	if (Blob.pix_basis)
	{
		area = GeoPar.area;
	}
	else
	{
		area = Blob.area;
	}

	idelta = (INTEGER) (MAPEM.delta);

	//------ set image to average density and initialize the work area for
	//       the total length of all projections through each pixel
	//cvd$  nodepchk

	for (i = 0; i < area; i++)
	{
		MAPEM.Wjj[i] = 0.0;
		if (Blob.pix_basis)
			MAPEM.Sjjinv[i] = 1 / getSjj(i, GeoPar.nelem); //blob case will not use Sjj Hstau 6/25/2003
	}

	if (Blob.pix_basis)
	{ //not needed in the blob case Hstau 6/3/2003
		for (i = 0; i < area; i++)
		{
			MAPEM.Kx[i] = 0.0;
			MAPEM.Sx[i] = 0.0;
		}
	}

	// compute the total length of all projections through each pixel

	for (np = 0; np < GeoPar.prjnum; np++)
	{
		for (first = 0; first < idelta; first++)
		{
			for (nr = first; nr < GeoPar.nrays; nr += idelta)
			{
				if (Blob.pix_basis)
				{
					wray(np, nr, list, weight, &numb, &snorm);
				}
				else
				{
					Blob.bwray(np, nr, list, weight, &numb, &snorm);
				}

				for (i = 0; i < numb; i++)
				{

					if (MAPEM.gamma1 > 0.0)
					{  //when blobs, gamma1 = 0.0
						MAPEM.Wjj[list[i]] += weight[i] / MAPEM.gamma1
								* MAPEM.Sjjinv[list[i]];
					}
					else
					{
						MAPEM.Wjj[list[i]] += weight[i];
					}
				}
			}
		}
	}

	//------ get projection data

	testneg = 0;
	for (np = 0; np < GeoPar.prjnum; np++)
	{
		for (usnr = 0; usnr < GeoPar.usrays; usnr++)
		{

			//nr = usnr + GeoPar.fusray - 1;
			nr = usnr + GeoPar.fusray; // mk

			// T.K. Narayan's modification -  24.Apr.1996
			// setting negative projection data to 0.0

			if (Anglst.prdta(np, nr) >= 0.0)
			{
				//MAPEM.prj[(np-1)*GeoPar.usrays+usnr] = prdta(np,nr);
				MAPEM.prj[(np) * GeoPar.usrays + usnr] = Anglst.prdta(np, nr);
			}
			else
			{
				//MAPEM.prj[(np-1)*GeoPar.usrays+usnr] = 0.0;
				MAPEM.prj[(np) * GeoPar.usrays + usnr] = 0.0;
				testneg++;
				if (testneg == 1)
				{
					fprintf(output,
							"\n     *** WARNING - There exist negative values in the projection data ***"); //bug 127, hstau 7/8/2005
					fprintf(output,
							"\n     *** Negative values set to 0.0 ***");
				}
			}
		}
	}

	return;
}

REAL getSjj(INTEGER j, INTEGER nelem)
// integer j, nelem
{
	INTEGER jx, jy;

	j++; // MK

	if (nelem == 3)
	{
		if (j == 5)
			return 1.;
		else
			return 1 / 64.;
	}                                //add exception when nelem=3,by wei, 1/2005

	jx = j - ((j - 1) / nelem) * nelem;
	jy = (j - 1) / nelem + 1;
	if ((jx > 2) && (jx < nelem - 1))
	{
		if ((jy > 2) && (jy < nelem - 1))
		{
			return 9 / 8.;
		}
		else
		{
			if ((jy == 2) || (jy == nelem - 1))
			{
				return 69 / 64.;
			}
			else
			{
				return 3 / 64.;
			}
		}
	}
	else
	{
		if ((jx == 2) || (jx == nelem - 1))
		{
			if ((jy > 2) && (jy < nelem - 1))
			{
				return 69 / 64.;
			}
			else
			{
				if ((jy == 2) || (jy == nelem - 1))
				{
					return 67 / 64.;
				}
				else
				{
					return 1 / 32.;
				}
			}
		}
		else
		{
			if ((jy > 2) && (jy < nelem - 1))
			{
				return 3 / 64.;
			}
			else
			{
				if ((jy == 2) || (jy == nelem - 1))
				{
					return 1 / 32.;
				}
				else
				{
					return 1 / 64.;
				}
			}
		}
	}
}
