/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/foru_unif.cpp $
 $LastChangedRevision: 85 $
 $Date: 2014-07-02 16:07:08 -0400 (Wed, 02 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 PURPOSE: FOR EACH CASE (ICASE=1,...,6) FIND ALL POINTS LIE BETWEEN
 THE TWO RAYS AND CALL UINT TO CALAULATE INTERPOLATED
 VALUE
 */

#include <cstdio>
#include <cmath>

#include "blkdta.h"
#include "fourie.h"
#include "consts.h"

#include "foru.h"

#define  cot(z)  (cos(z)/sin(z));

void foru_class::unif(INTEGER icase, REAL ang1, REAL ang2, REAL* nold, REAL* new1)
{
	INTEGER kh, ix, iy, j, khd, kk;
	REAL uleft, uright, uhigh, ulow, xhigh, xlow, yhigh, ylow;

	kh = Fourie.nsize2 / 2 - 1;

	switch (icase)
	{

//.... CASE 1
	case 1:
		uleft = (REAL) cot(ang1)
		;
		uright = (REAL) cot(ang2)
		;
		for (j = 1; j <= kh; j++)
		{
			iy = -j;
			xlow = uleft * iy - Fourie.epsln;
			xhigh = uright * iy + Fourie.epsln;
			ix = lround(xlow); // changed "(INTEGER)" to "lround" to match Fortran code. Lajos, Jan 20, 2005
			while (ix <= xhigh)
			{
				if (ix >= xlow)
				{
					uint1(icase, ix, iy, ang1, ang2, nold, new1);
				}
				ix++;
			}
		}
		break;

//.... CASE 2
	case 2:
		uleft = (REAL) cot(ang1)
		;
		for (j = 1; j <= kh; j++)
		{
			iy = -j;
			xlow = uleft * iy - Fourie.epsln;
			ix = lround(xlow); // changed "(INTEGER)" to "lround" to match Fortran code. Lajos, Jan 20, 2005
			while (ix <= j)
			{
				if (ix >= xlow)
				{
					uint1(icase, ix, iy, ang1, ang2, nold, new1);
				}
				ix++;
			}
		}

		uhigh = (REAL) tan(ang2);
		for (ix = 1; ix <= kh; ix++)
		{
			yhigh = ix * uhigh + Fourie.epsln;
			iy = -ix + 1;
			while (iy <= yhigh)
			{
				uint1(icase, ix, iy, ang1, ang2, nold, new1);
				iy++;
			}
		}
		break;

		// CASE 3
	case 3:
		ulow = (REAL) tan(ang1);
		uhigh = (REAL) tan(ang2);
		for (ix = 1; ix <= kh; ix++)
		{
			ylow = ulow * ix - Fourie.epsln;
			yhigh = uhigh * ix + Fourie.epsln;
			iy = lround(ylow); // changed "(INTEGER)" to "lround" to match Fortran code. Lajos, Jan 20, 2005
			while (iy <= yhigh)
			{
				if (iy >= ylow)
				{
					uint1(icase, ix, iy, ang1, ang2, nold, new1);
				}
				iy++;
			}
		}
		break;

		// CASE 4

	case 4:
		ulow = (REAL) tan(ang1);
		for (ix = 1; ix <= kh; ix++)
		{  // !!! ix = 0 does not work mk
			ylow = ulow * ix - Fourie.epsln;
			iy = lround(ylow); // changed "(INTEGER)" to "lround" to match Fortran code. Lajos, Jan 20, 2005
			while (iy <= ix)
			{ // changed ">=" to "<=" to match Fortran code. Lajos, Jan 13, 2005
				if (iy >= ylow)
				{
					uint1(icase, ix, iy, ang1, ang2, nold, new1);
				}
				iy++;
			}
		}

		uleft = (REAL) cot(ang2)
		;

		for (iy = 1; iy <= kh; iy++)
		{
			xlow = uleft * iy - Fourie.epsln;
			ix = iy - 1;
			while (ix >= xlow)
			{
				uint1(icase, ix, iy, ang1, ang2, nold, new1);
				ix--;
			}
		}

		break;

		// CASE 5

	case 5:
		uleft = (REAL) cot(ang2)
		;
		uright = (REAL) cot(ang1)
		;
		for (iy = 1; iy <= kh; iy++)
		{
			xlow = uleft * iy - Fourie.epsln;
			xhigh = uright * iy + Fourie.epsln;
			ix = lround(xlow); // changed "(INTEGER)" to "lround" to match Fortran code. Lajos, Jan 20, 2005
			while (ix <= xhigh)
			{
				if (ix >= xlow)
				{
					uint1(icase, ix, iy, ang1, ang2, nold, new1);
				}
				ix++;
			}
		}
		break;

		// CASE 6

		// CONJUGATE THE SECOND PROJECTION
	case 6:
		khd = Fourie.nsize1 + 2;
		for (kk = 1; kk < khd; kk += 2)
		{
			new1[kk] = -new1[kk];
		}

		uleft = (REAL) cot(ang2)
		;
		uright = (REAL) cot(ang1)
		;
		for (iy = 1; iy <= kh; iy++)
		{
			xlow = uleft * iy - Fourie.epsln;
			xhigh = uright * iy + Fourie.epsln;
			ix = lround(xlow); // changed "(INTEGER)" to "lround" to match Fortran code. Lajos, Jan 20, 2005
			while (ix <= xhigh)
			{
				if (ix >= xlow)
				{
					uint1(icase, ix, iy, ang1, ang2 + Consts.pi, nold, new1);
				}
				ix++;
			}
		}

		// CHANGE BACK

		khd = Fourie.nsize1 + 2;
		for (kk = 1; kk < khd; kk += 2)
		{
			new1[kk] = -new1[kk];
		}
		break;
	}

	return;
}
