/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/foru_vcase1.cpp $
 $LastChangedRevision: 85 $
 $Date: 2014-07-02 16:07:08 -0400 (Wed, 02 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 ------------------------------------------------------------------

 VCASE1: VARIABLE SPACING CASE 1
 PURPOSE: FOR VARIABLE SPACING, ICASE=1 (THE TWO LINES LIE BETWEEN
 -90 AND -45 DEGREE LINES) FIND ALL POINTS THAT LIE
 BETWEEN THE TWO LINES AND INTERPOLATE

 ------------------------------------------------------------------
 */
// bug79 - problem with fourier algorithm
#include <cstdio>
#include <cmath>

#include "blkdta.h"
#include "fourie.h"
#include "consts.h"

#include "foru.h"

#define  cot(z)  (cos(z)/sin(z));

void foru_class::vcase1(REAL ang1, REAL ang2, REAL* pj1, REAL* pj2)
{
	int j;

	INTEGER iy;
	INTEGER indl;
	INTEGER indh;
	REAL a;
	INTEGER low;
	INTEGER indmin;
	INTEGER indmax;
	REAL distan;
	REAL yg;
	INTEGER ix;
	REAL xg;
	REAL gr;
	REAL gi;
	REAL amid;
	INTEGER ind;
	INTEGER x;
	REAL t;
	REAL pr;
	REAL px;
	REAL qr;
	REAL qi;
	REAL s;

	REAL xx[4];
	REAL yy[4];

	REAL uleft;
	REAL uright;
	INTEGER kh;

	REAL aleft;
	REAL aright;

	uleft = (REAL) cot(ang1);
	uright = (REAL) cot(ang2);
	kh = Fourie.nsize2 / 2 - 1;

	for (j = 1; j <= kh; j++)
	{
		// bug79 - swr 2/11/05

		iy = -j;
		aleft = uleft * iy;
		aright = uright * iy;
		indl = (INTEGER) ceil(aleft - Fourie.epsln); // bug82 - changed to ceil - swr - 2/20/05
		indh = (INTEGER) floor(aright + Fourie.epsln); // bug82 - changed to floor - swr - 2/20/05
		if (indl <= indh)
		{
			a = j * Fourie.delp / Fourie.deld;
			low = (INTEGER) a;
			indmin = 2 * low;     // bug79 - swr 2/11/05
			indmax = indmin + 2;
			distan = aright - aleft;

			switch (Fourie.interp)
			{

			// INTERP = 1 NEAREST NEIGHBOR IN CARTESIAN SENSE
			// INTERP = 4 F=(F1/D1+F2/D2+F3/D3+F4/D4)/(1/D1+1/D2+1/D3+1/D4)

			case 1:
			case 4:
				yy[1] = (REAL) -low;
				yy[3] = (REAL) -low;
				yy[2] = (REAL) -low - 1;
				yy[4] = (REAL) -low - 1;
				xx[1] = yy[1] * uleft;
				xx[2] = yy[2] * uleft;
				xx[3] = yy[3] * uright;
				xx[4] = yy[4] * uright;
				yg = -a;
				for (ix = indl; ix <= indh; ix++)
				{     // bug79 - swr 2/11/05
					xg = ix * Fourie.delp / Fourie.deld;
					vint14(xx, yy, xg, yg, pj1, pj2, indmin, indmax, &gr, &gi); // changed to pass address of "gr" and "gi". Lajos, Feb 10, 2005
					store(ix, iy, gr, gi);
				}

				break;      // bug79 - swr 2/15/05

				// INTERP = 2 NEAREST NEIGHBOR IN RADIAL SENSE

			case 2:
				amid = (aleft + aright) / (REAL) 2.0;
				ind = 2 * (lround(a)) + 1; // changed "(INTEGER)" to "lround" to match Fortran code. Lajos, Jan 20, 2005
				for (ix = indl; ix <= indh; ix++)
				{     // bug79 - swr 2/11/05
					x = ix;
					if (x <= amid)
					{      // bug79 - swr 2/15/05
						gr = pj1[ind - 1];
						gi = pj1[ind];
					}
					else
					{
						gr = pj2[ind - 1];
						gi = pj2[ind];
					}
					store(ix, iy, gr, gi);
				}
				break;

				// INTERP = 3 LINEAR INTERPOLATION
			case 3:
				t = a - low;
				if (t <= Consts.zero)
					t = 0.0;
				if ((1. - t) <= Consts.zero)
					t = 1.0;
				pr = pj1[indmin] * ((REAL) 1.0 - t) + pj1[indmax] * t;
				px = pj1[indmin + 1] * ((REAL) 1.0 - t) + pj1[indmax + 1] * t;
				qr = pj2[indmin] * ((REAL) 1.0 - t) + pj2[indmax] * t;
				qi = pj2[indmin + 1] * ((REAL) 1.0 - t) + pj2[indmax + 1] * t;

				for (ix = indl; ix <= indh; ix++)
				{     // bug79 - swr 2/11/05
					s = (ix - aleft) / distan;
					if (s <= Consts.zero)
						s = 0.0;
					if ((1.0 - s) <= Consts.zero)
						s = 1.0;
					gr = pr * ((REAL) 1.0 - s) + qr * s;
					gi = px * ((REAL) 1.0 - s) + qi * s;
					store(ix, iy, gr, gi);
				}
				break;
			}
		}
	}
}
