/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/get_iter_flag.c $
 $LastChangedRevision: 137 $
 $Date: 2014-07-15 16:38:33 -0400 (Tue, 15 Jul 2014) $
 $Author: bprommegger $
 ***********************************************************

 Previously part of SuperSNARK
 */

#include <string.h>
#include "experimenter.h"
#include "errorc.h"
#include "get_iter_flag.h"

/* --------------------------- get_iter_flag.c ---------------------------

 This function reads the list of iteration numbers stored in the
 file containing the comparisons to be made and generates an iteration_flag_line
 which is needed for the EVALUATE command.

 INPUTS:
 anaflname - name of file containing comparisons to be made.
 iter_flag - string containing a 1 followed by 50 0's.

 OUTPUTS:
 iter_flag - iteration_flag_line needed for the EVALUATE command
 ------------------------------------------------------------------     */

void getiterflag(char* anaflname, char* iter_flag)
{
	int iter1, iter2, bit1, bit2, i;
	char string[MAXLINESIZE];
	char flag = '1';

	FILE *anafl;

	if ((anafl = fopen(anaflname, "r")) == NULL)
	{
		errorc("in get_iter_flag.c: error in opening file", anaflname);
	}

	/* the iter_flag is the union of all the iteration numbers */

	while (fgets(string, sizeof(string), anafl) != NULL)
	{
		// upgrade flag to 3 if klds is specified
		if (!isSkip(string))
		{
			if (strcasestr(string, "klds")) flag='3';
			if (strcasestr(string, "wsqd")) flag='3';
		}

		if (sscanf(string, "%d %d", &iter1, &iter2) == 2)
		{
			/* if an iteration number is a multiple of 50
			 set bit 50 */
			bit1 = iter1 % 50;
			bit2 = iter2 % 50;
			if (bit1 == 0 && iter1 != 0)
			{
				iter_flag[50] = flag;
			}
			else
			{
				iter_flag[bit1] = flag;
			}

			if (bit2 == 0 && iter2 != 0)
			{
				iter_flag[50] = flag;
			}
			else
			{
				iter_flag[bit2] = flag;
			}
		}
	}

	iter_flag[51] = '\0';
	fclose(anafl);
}
