/*
***********************************************************
$SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
$HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/nav2.cpp $
$LastChangedRevision: 145 $
$Date: 2014-07-17 19:49:25 -0400 (Thu, 17 Jul 2014) $
$Author: olangthaler $
***********************************************************

*********************************************************************************
Non-ascending vector for secondary optimization criterion "smoothness" (sec_cri2)
*********************************************************************************
*/

#include <cstdio>
#include <stdlib.h>
#include <string.h>
#include <iostream>
#include "nav2.h"
#include "math.h"
#include "anglst.h"
#include "blob.h"

void nav2_class::Run(REAL* xkn, REAL* vkn)
{
	int pixelcount = GeoPar.area;

	REAL gradientSquareNorm = 0;
	calcGradient(xkn, vkn);
	for (INTEGER i = 0; i < pixelcount; i++) gradientSquareNorm += pow(vkn[i], 2);
	REAL gradientNorm = sqrt(gradientSquareNorm);
	if (gradientNorm > pow(10,-15))	for (INTEGER i = 0; i < pixelcount; i++) vkn[i] = (-1.0) * vkn[i] / gradientNorm;
	else for (INTEGER i = 0; i < pixelcount; i++) vkn[i] = 0;
}

void nav2_class::calcGradient(REAL* x, REAL* grad)
{
	// read image dimensions
	int nelem = GeoPar.nelem;
	int pixelcount = GeoPar.area;
	double temp=0;

	// define minimum value for divisions
	double min = pow(10,-15);

	// set gradient starting value
	memset(grad,0,pixelcount*sizeof(REAL));

	// pre-processing
	REAL* pre = new REAL[pixelcount];
	memset(pre,0,pixelcount*sizeof(REAL));

	// iterate through all pixels that are not on the border
	for (int i=nelem+1; i<pixelcount-nelem-1; i++)
	{
		// skip left and right column
		if (!(i%nelem) || !((i+1)%nelem)) continue;

		pre[i]=x[i]-(0.125*(x[i-nelem-1]+x[i-nelem]+x[i-nelem+1]+x[i-1]+x[i+1]+x[i+nelem-1]+x[i+nelem]+x[i+nelem+1]));
	}

	// actual processing
	// the four corners
	temp=-0.25*pre[nelem+1];
	if (fabs(temp)>min) grad[0]=temp;
	temp=-0.25*pre[2*nelem-2];
	if (fabs(temp)>min) grad[nelem-1]=temp;
	temp=-0.25*pre[pixelcount-2*nelem+1];
	if (fabs(temp)>min) grad[pixelcount-nelem]=temp;
	temp=-0.25*pre[pixelcount-nelem-2];
	if (fabs(temp)>min) grad[pixelcount-1]=temp;

	// the 8 pixels adjoining the corners
	temp=-0.25*(pre[nelem+1]+pre[nelem+2]);
	if (fabs(temp)>min) grad[1]=temp;
	temp=-0.25*(pre[nelem+1]+pre[2*nelem+1]);
	if (fabs(temp)>min) grad[nelem]=temp;

	temp=-0.25*(pre[2*nelem-2]+pre[2*nelem-3]);
	if (fabs(temp)>min) grad[nelem-2]=temp;
	temp=-0.25*(pre[2*nelem-2]+pre[3*nelem-2]);
	if (fabs(temp)>min) grad[2*nelem-1]=temp;

	temp=-0.25*(pre[pixelcount-2*nelem+1]+pre[pixelcount-2*nelem+2]);
	if (fabs(temp)>min) grad[pixelcount-nelem+1]=temp;
	temp=-0.25*(pre[pixelcount-2*nelem+1]+pre[pixelcount-3*nelem+1]);
	if (fabs(temp)>min) grad[pixelcount-2*nelem]=temp;

	temp=-0.25*(pre[pixelcount-nelem-2]+pre[pixelcount-nelem-3]);
	if (fabs(temp)>min) grad[pixelcount-2]=temp;
	temp=-0.25*(pre[pixelcount-nelem-2]+pre[pixelcount-2*nelem-2]);
	if (fabs(temp)>min) grad[pixelcount-nelem-1]=temp;

	// the remainder of the top and bottom rows
	int t=0;
	for (int i=2; i<nelem-2; i++)
	{
		temp=-0.25*(pre[i+nelem-1]+pre[i+nelem]+pre[i+nelem+1]);
		if (fabs(temp)>min) grad[i]=temp;

		t = i+(nelem*(nelem-1));
		temp=-0.25*(pre[t-nelem-1]+pre[t-nelem]+pre[t-nelem+1]);
		if (fabs(temp)>min) grad[t]=temp;
	}

	// the remainder of the left and right columns
	for (int i=2*nelem; i<nelem*(nelem-3)+1; i+=nelem)
	{
		temp=-0.25*(pre[i-nelem+1]+pre[i+1]+pre[i+nelem+1]);
		if (fabs(temp)>min) grad[i]=temp;

		t = i+nelem-1;
		temp=-0.25*(pre[t-nelem-1]+pre[t-1]+pre[t+nelem-1]);
		if (fabs(temp)>min) grad[t]=temp;
	}

	// the remainder of the image: all pixels that are not on the border
	// note: since the border pixels of the preprocessed array are 0, the
	// pixels of the "inner border" do not need to be handled separately
	for (int i=nelem+1; i<pixelcount-nelem-1; i++)
	{
		// skip left and right column
		if (!((i)%nelem) || !((i+1)%nelem)) continue;

		temp=2*pre[i]-0.25*(pre[i-nelem-1]+pre[i-nelem]+pre[i-nelem+1]+pre[i-1]+pre[i+1]+pre[i+nelem-1]+pre[i+nelem]+pre[i+nelem+1]);

		if (fabs(temp)>min) grad[i]=temp;
	}

	delete[] (pre);

	// failsafe
	for (int i=0;i<pixelcount;i++)
	{
		if (isnan(grad[i]))
		{
			grad[i]=0;
		}
	}
}
