/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/qinit.cpp $
 $LastChangedRevision: 80 $
 $Date: 2014-07-01 21:01:54 -0400 (Tue, 01 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 INITIALIZE FILTER FUNCTION TABLE
 FILTER = 1     FOR BAND LIMITING FILTER
 = 2     FOR LOW PASS SINC FILTER
 = 3     FOR LOW PASS COS  FILTER
 = 4     FOR GENERALIZED HAMMING FILTER
 (CUTOFF BECOMES PARAMETER ALPHA)
 */

#include <cstdlib>       
#include <cstdio>
#include <cmath>

#include "blkdta.h"
#include "sinc.h"
#include "qinit.h"
#include "consts.h" 
#include "uiod.h"

void qinit(REAL* table, INTEGER isize, INTEGER filter, REAL c)
{
	REAL mc;

	REAL factr;
	INTEGER i;
	REAL amc;
	REAL smc;
	REAL h1;
	REAL h2;
	REAL cm1;
	INTEGER nsize;
	INTEGER k;

	switch (filter)
	{
	// BRACEWELL BAND LIMITING FILTER

	case 0:
		factr = c * c / (REAL) 4.0;
		table[0] = factr;

		for (i = 1; i < isize; i++)
		{
			mc = i * c;
			table[i] = factr
					* ((REAL) 2.0 * sinc(mc) - SQR(sinc(mc / ((REAL) 2.0))));
		}
		return;

		// SHEPP LOW PASS FILTER

	case 1:
		factr = SQR(c / Consts.pi);
		table[0] = (REAL) 2.0 * factr;
		factr *= Consts.pid2;

		for (i = 1; i < isize; i++)
		{
			mc = i * c;
			amc = (REAL) 0.5 + mc;
			smc = (REAL) 0.5 - mc;
			table[i] = factr
					* (sinc((REAL) 0.5 * amc) * (REAL) cos(Consts.pid2 * smc)
							+ sinc((REAL) 0.5 * smc)
									* (REAL) cos(Consts.pid2 * amc));
		}
		return;

		// LOW PASS FILTER (COS)

	case 2:
		factr = SQR(c / Consts.pi);
		table[0] = factr * (Consts.pi - (REAL) 2.0);
		factr *= Consts.pid2 * Consts.pid2;

		for (i = 1; i < isize; i++)
		{
			mc = i * c;
			amc = (REAL) 0.5 + mc;
			smc = (REAL) 0.5 - mc;
			h1 = sinc((REAL) 0.5 * amc);
			h2 = sinc((REAL) 0.5 * smc);
			table[i] =
					factr
							* (sinc(amc) + sinc(smc)
									- (REAL) 0.5 * (SQR(h1) + SQR(h2)));
		}
		return;

		// GENERALIZED HAMMING

	case 3:
		cm1 = c - (REAL) 1.0;
		factr = (REAL) 1.0 / (Consts.pi * Consts.pi);
		table[0] = c / (REAL) 4.0 + cm1 * factr;
		table[1] = -c * factr - cm1 / (REAL) 8.0;
		nsize = isize - 1;

		for (k = 2; k < nsize; k += 2)
		{
			table[k] = cm1 * factr / (REAL) 2.0
					* ((REAL) pow((k - 1), (-2)) + (REAL) pow((k + 1), (-2)));
			table[k + 1] = -c * factr * (REAL) pow((k + 1), (-2));
		}

		table[isize - 1] = cm1 * factr / (REAL) 2.0
				* ((REAL) pow((isize - 2), (-2)) + (REAL) pow((isize), (-2)));
		return;

	default:
		fprintf(output, "\n **** filter = %d is not a valid filter type for qinit routine", filter);
		fprintf(output, "\n **** program aborted\n");
		exit(500);
		//}
	}
}
