/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/quad_adsmos.cpp $
 $LastChangedRevision: 80 $
 $Date: 2014-07-01 21:01:54 -0400 (Tue, 01 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 THIS ROUTINE PERFORMS THE FOUR DIFFERENT SMOOTHING PROCEDURES
 USED IN THE QUADRATIC OPTIMIZATION ALGORITHM.
 */

#include <cstdio>

#include "blkdta.h"
#include "geom.h"

#include "blob.h"
#include "quad.h"

//pragma message( "to fix!" )
/// fix indexes 1 -> 0

void quad_class::adsmos(REAL* x, INTEGER m, REAL w1, REAL w2, REAL w3,
		INTEGER bcon)
{

	BOOLEAN snar, sajc;

	static const INTEGER hsnar = CHAR2INT('s', 'n', 'a', 'r');
	static const INTEGER hsajc = CHAR2INT('s', 'a', 'j', 'c');
	static const INTEGER hkami = CHAR2INT('k', 'a', 'm', 'i');

	INTEGER kami;
	INTEGER n;
	INTEGER n1;
	INTEGER n2;
	REAL weit4;
	REAL weit6;
	REAL weit9;
	REAL* ip = NULL;
	REAL* in = NULL;   //wei  3/2005
	// REAL* ipf;          //wei 3/2005
	INTEGER j;
	INTEGER i;
	INTEGER l1;
	INTEGER ln;
	INTEGER k;
	REAL* ipin;
	INTEGER li;
	INTEGER l;

	// FOR THE KAMI METHOD THE ROUTINE IS EXECUTED TWICE

	kami = 1;

	// INDEX PARAMETERS

	n = GeoPar.nelem;
	n1 = n - 1;
	n2 = n + 1;
	snar = FALSE;
	sajc = FALSE;
	if (bcon == hsnar)
		snar = TRUE;
	if (bcon == hsajc)
		sajc = TRUE;
	if (snar)
	{
		weit4 = (REAL) (w1 + 2.0 * w2 + w3);
		weit6 = weit4 + w2 + w3;
		weit9 = (REAL) (weit6 + w2 + 2.0 * w3);

	}
	// ALLOCATE WORKING SPACE

	ip = new REAL[n];

	in = new REAL[n];

	// IPF IS  USE FOR SUB FREE ONLY

	//ipf = ip;         wei, 3/2005

	// SMOOTHING IS DONE M TIMES

	for (;;)
	{
		for (j = 0; j < m; j++)
		{

			// SMOOTH FIRST ROW OF PICTURE

			in[0] = x[0];
			in[n1] = x[n1];

			if (!sajc)
			{

				x[0] = w1 * x[0] + w2 * (x[1] + x[n]) + w3 * x[n2];
				if (snar)
					x[0] /= weit4;

				for (i = 1; i < n1; i++)
				{

					in[i] = x[i];

					x[i] = w1 * x[i] + w2 * (in[i - 1] + x[i + 1] + x[n + i])
							+ w3 * (x[n1 + i] + x[n2 + i]);

					if (snar)
						x[i] /= weit6;
				}
				x[n1] = w1 * x[n1] + w2 * (in[n1 - 1] + x[n1 + n])
						+ w3 * x[n + n1 - 1];

				if (snar)
					x[n - 1] /= weit4;
			}
			else
			{
				x[0] = (REAL) (w1 * x[0] + w2 * (2.0 * x[0] + x[1] + x[n])
						+ w3 * (x[0] + x[1] + x[n2] + x[n]));

				for (i = 1; i < n1; i++)
				{
					in[i] = x[i];

					x[i] = w1 * x[i] + (w2 + w3) * (in[i - 1] + x[i + 1])
							+ w2 * (x[i] + x[n + i])
							+ w3 * (x[n1 + i] + x[n2 + i]);

				}

				x[n1] = (REAL) (w1 * x[n1]
						+ w2 * (in[n1 - 1] + 2. * x[n1] + x[n + n1])
						+ w3 * (x[n1] + in[n1 - 1] + x[n1 + n] + x[n1 + n1]));
			}

			// ROW 2 TO ROW N - 1

			l = n;
			//l1 = n2;
			l1 = n;
			//ln = 2*n;
			ln = 2 * n - 1;

			for (k = 1; k < n1; k++)
			{
				ipin = in;
				in = ip;
				ip = ipin;
				in[0] = x[l1];
				in[n1] = x[ln];
				x[l1] = w1 * x[l1] + w2 * (ip[0] + x[l1 + 1] + x[n + l1])
						+ w3 * (ip[1] + x[l1 + n2]);

				if (sajc)
					x[l1] += w2 * (in[0]) + w3 * (ip[0] + x[l1 + n1]);
				if (snar)
					x[l1] /= weit6;

				for (i = 1; i < n1; i++)
				{

					li = l + i;
					in[i] = x[li];

					x[li] = w1 * x[li]
							+ w2 * (in[i - 1] + x[li + 1] + ip[i] + x[li + n])
							+ w3
									* (ip[i - 1] + ip[i + 1] + x[li + n1]
											+ x[li + n2]);

					if (snar)
						x[li] /= weit9;
				}

				x[ln] = w1 * x[ln] + w2 * (in[n - 2] + ip[n - 1] + x[ln + n])
						+ w3 * (ip[n - 2] + x[ln + n1]);

				if (snar)
					x[ln] /= weit6;
				if (sajc)
					x[ln] += w2 * in[n1] + w3 * (ip[n1] + x[ln + n]);

				l = l + n;
				l1 = l1 + n;
				ln = ln + n;
			}

			// SMOOTH LAST ROW

			// IN LAST ROW CHANGE ROLE OF IN AND IP

			//ip[1] = x[l1];
			ip[0] = x[l1];

			if (!sajc)
			{

				x[l1] = w1 * x[l1] + w2 * (in[0] + x[l1 + 1]) + w3 * (in[1]);

				if (snar)
					x[l1] /= weit4;

				for (i = 1; i < n1; i++)
				{
					li = l + i;
					ip[i] = x[li];

					x[li] = w1 * x[li] + w2 * (in[i] + ip[i - 1] + x[li + 1])
							+ w3 * (in[i - 1] + in[i + 1]);

					if (snar)
						x[li] /= weit6;
				}

				x[ln] = w1 * x[ln] + w2 * (ip[n1 - 1] + in[n1])
						+ w3 * in[n1 - 1];

				if (snar)
					x[ln] /= weit4;
			}
			else
			{
				x[l1] = (REAL) (w1 * x[l1]
						+ w2 * (2.0 * x[l1] + in[0] + x[l1 + 1])
						+ w3 * (x[l1] + in[1] + in[0] + x[l1 + 1]));

				for (i = 1; i < n1; i++)
				{
					li = l + i;

					ip[i] = x[li];

					x[li] = w1 * x[li] + (w2 + w3) * (ip[i - 1] + x[li + 1])
							+ w2 * (in[i] + x[li])
							+ w3 * (in[i - 1] + in[i + 1]);

				}

				x[ln] = (REAL) (w1 * x[ln]
						+ w2 * (in[n1] + ip[n1 - 1] + 2.0 * x[ln])
						+ w3 * (x[ln] + in[n - 1] + in[n1 - 1] + ip[n1 - 1]));
			}
		}

		if ((bcon != hkami) || (kami == 2))
			goto L17;

		// PREPARE FOR SECOND ROUND FOR KAMI

		kami = 2;

		for (i = 0; i < n; i++)
		{
			x[i] = 0.0;
			x[l + i] = 0.0;
		}

		l1 = 0;
		ln = n;
		for (i = 2; i < n; i++)
		{
			l1 = l1 + n;
			ln = ln + n;
			x[l1] = 0.0;
			x[ln] = 0.0;
		}
	}

	L17:
	//delete[] ipf;  // bug 92 - Lajos - 03/02/2005   wei,3/2005
	delete[] ip;
	delete[] in;  // wei 3/2005

	return;
}
