/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/quad_blurck.cpp $
 $LastChangedRevision: 80 $
 $Date: 2014-07-01 21:01:54 -0400 (Tue, 01 Jul 2014) $
 $Author: agulati $
 ***********************************************************
 */

#include <cstdio>

#include "blkdta.h"
#include "consts.h"
#include "uiod.h"
#include "int2str.h"

#include "quad.h"

void quad_class::blurck(REAL b1, REAL b2, REAL b3, INTEGER bcon, INTEGER npow,
		BOOLEAN* alg1)
{

	static const INTEGER hsnar = CHAR2INT('s', 'n', 'a', 'r');
	static const INTEGER hkami = CHAR2INT('k', 'a', 'm', 'i');
	static const INTEGER hsajz = CHAR2INT('s', 'a', 'j', 'z');
	static const INTEGER hsajc = CHAR2INT('s', 'a', 'j', 'c');

	REAL bsum;
	BOOLEAN psd;
	BOOLEAN pos;

	if (!((bcon == hsnar) || (bcon == hkami) || (bcon == hsajz)
			|| (bcon == hsajc)))
	{
		*alg1 = TRUE;
		fprintf(output,
				"\n\n      ***unrecognized parameter %4s, program stops***",
				int2str(bcon));

		return;
	}
	bsum = b1 + (REAL) 4.0 * (b2 + b3);
	fprintf(output, "\n\n      variance of density in an arbitrary pixel is  %9.5f", b1);
	fprintf(output, "\n       covariance of densities in neighboring pixels is %9.5f", b2);
	fprintf(output, "\n       covariance of densities in diagonally neighboring pixels is %9.5f", b3);

	fprintf(output, "\n\n      %4s boundary conditions", int2str(bcon));
	fprintf(output, "\n       sum of weights %9.4f", bsum);

	if (!((bcon != hkami) || (npow == 1)))
	{
		fprintf(output,
				"\n\n      ***for kami only first power is allowed, program stops***");

		*alg1 = TRUE;
		return;
	}
	if (bcon == hkami)
		return;

	// CHECKING ON POSITIVE-DEFINITENESS OF SMOOTHING MATRIX
	psd = TRUE;
	if (bsum <= -.000001)
		psd = FALSE;
	if ((b1 - 4.0 * b2 + 4.0 * b3) <= -.000001)
		psd = FALSE;
	if ((b1 - 4.0 * b3) <= -.000001)
		psd = FALSE;
	*alg1 = !psd;
	if (*alg1)
	{
		fprintf(output,
				"\n\n      ***warning, smoothing matrix is not positive semi definite");
	}
	if (bcon != hsnar)
		return;

//  PERFORMING TRADITIONAL SNARK CHECK ON POSITIVITY
	pos = FALSE;
	if ((b1 > Consts.zero) && (b2 >= 0.0) && (b3 >= 0.0))
		pos = TRUE;
	*alg1 = !pos;
	if (*alg1)
	{
		fprintf(output, "\n\n      ***elements of smoothing matrix too small or negative, program stops***"); //hstau
	}
	return;
}
