/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/quad_costfn.cpp $
 $LastChangedRevision: 80 $
 $Date: 2014-07-01 21:01:54 -0400 (Tue, 01 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 THIS SUBROUTINE COMPUTES THE VALUE OF K(X) NEEDED BY THE QUADRATIC
 OPTIMIZATION ALGORITHM.
 */

#include <cstdio>
#include <cmath>

#include "blkdta.h"
#include "geom.h"
#include "fourie.h"
#include "consts.h"
#include "uiod.h"
#include "wray.h"
#include "ray.h"
#include "anglst.h"
#include "errfac.h"
#include "blob.h"

#include "quad.h"

void quad_class::costfn(REAL* recon, REAL* r, REAL* g, REAL* cg, REAL* w,
		INTEGER* list, REAL* weight, REAL* f)
{

	static REAL ppcost; // made static to avoid garbage value for `iterx > 1'. Lajos, Feb 10, 2005
	INTEGER np;
	REAL theta;
	REAL sinth;
	REAL costh;
	REAL rinc2;
	INTEGER nr;
	REAL pr;
	BOOLEAN alg;
	REAL xxcost;
	REAL bxcost;
	INTEGER i;
	REAL xmcost;
	REAL pmcost;
	INTEGER numb;
	REAL snorm;
	REAL val;
	INTEGER nb;
	INTEGER k;
	REAL valder;
	REAL cost;

	INTEGER area;
	if (Blob.pix_basis)
	{
		area = GeoPar.area;
	}
	else
	{
		area = Blob.area;
	}

	if (copt == 4)
	{
		fprintf(output, "\n\n      cost function can not be computed");
		return;
	}

	if (iterx <= 1)
	{
		ppcost = 0.0;
		Fourie.rinc = GeoPar.pinc;
		for (np = 0; np < GeoPar.prjnum; np++)
		{
			Anglst.getang(np, &theta, &sinth, &costh);
			if (GeoPar.vri)
				Fourie.rinc = GeoPar.pinc
						* MAX0((REAL) fabs(costh), (REAL) fabs(sinth));
			rinc2 = Fourie.rinc * Fourie.rinc;
			for (nr = 0; nr < GeoPar.nrays; nr++)
			{
				pr = Anglst.prdta(np, nr);
				if ((eopt - 2) >= 0)
				{
					if ((eopt - 2) > 0)
					{
						error_1 = uerror(np, nr, pr, &alg);
					}
					else
					{
						error_1 = errfac(pr, Fourie.rinc, rinc2);
					}
				}
				ppcost += pr * pr / error_1;
			}
		}

		n = w;
		if (w == r)
		{
			n = new REAL[area];
		}
		m = n;

		if (!((sb < Consts.zero) || (l <= 0)))
		{
			m = cg;
			if (cg == g)
			{
				m = new REAL[area];
			}
		}
	}

	xxcost = 0.0;
	bxcost = 0.0;
	if ((sb + sc) > Consts.zero)
	{
		for (i = 0; i < area; i++)
		{
			if (fopt >= 3)
				xbar = f[i];
			n[i] = recon[i] - xbar;
			m[i] = n[i];
		}

		if ((sb > Consts.zero) && (l > 0))
		{
			if (Blob.pix_basis)
			{
				adsmos(n, l, bw1, bw2, bw3, bbcon);
			}
			else
			{
				badsmos(n, l, bw1, bw2, bbcon);
			}
		}

		for (i = 0; i < area; i++)
		{
			if (Blob.pix_basis || (!Blob.pix_basis && i % 2 == Blob.pr))
			{
				xxcost += m[i] * m[i];
				if (sb > Consts.zero)
					bxcost += m[i] * n[i];
			}
		}
	}

	xmcost = 0.0;
	pmcost = 0.0;
	Fourie.rinc = GeoPar.pinc;
	for (np = 0; np < GeoPar.prjnum; np++)
	{
		Anglst.getang(np, &theta, &sinth, &costh);
		if (GeoPar.vri)
			Fourie.rinc = GeoPar.pinc * MAX0((REAL) fabs(costh), (REAL) fabs(sinth));
		rinc2 = Fourie.rinc * Fourie.rinc;
		for (nr = 0; nr < GeoPar.nrays; nr++)
		{
			pr = Anglst.prdta(np, nr);
			if (GeoPar.strip)
			{
				if (Blob.pix_basis)
				{
					ray(np, nr, list, weight, &numb, &snorm);
				}
				else
				{
					Blob.bray(np, nr, list, weight, &numb, &snorm);
				}
			}
			if (GeoPar.line)
			{
				if (Blob.pix_basis)
				{
					wray(np, nr, list, weight, &numb, &snorm);
				}
				else
				{
					Blob.bwray(np, nr, list, weight, &numb, &snorm);
				}
			}
			if (numb != 0)
			{
				val = 0.0;
				for (nb = 0; nb < numb; nb++)
				{
					k = list[nb];
					val += recon[k] * weight[nb];
				}

				if ((eopt - 2) >= 0)
				{
					if ((eopt - 2) > 0)
					{
						error_1 = uerror(np, nr, pr, &alg);
					}
					else
					{
						error_1 = errfac(pr, Fourie.rinc, rinc2);
					}
				}
				valder = val / error_1;
				xmcost += val * valder;
				pmcost += pr * valder;
			}
		}
	}

	cost = sa * (ppcost - (REAL) 2.0 * pmcost + xmcost) + sb * bxcost + sc * xxcost;
	fprintf(output, "\n\n      value of cost function %15.6e", cost);

	return;
}
