/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/quad_deltac.cpp $
 $LastChangedRevision: 80 $
 $Date: 2014-07-01 21:01:54 -0400 (Tue, 01 Jul 2014) $
 $Author: agulati $
 ***********************************************************

                                 R#*R
        DELTA = ---------------------------------------------------
                G#*C**T*((SA*M#*A*M + SB*B**L)*C**T + SC*C**(T-1))*G

    AOPT=1             AOPT=2            AOPT=3           AOPT=4
    (R=G=V)            (R=G)             (R=V)

1.                                       PRTNRM=TWONRM    SAME
2.  TWONRM=R#*R        SAME              SAME             SAME
3.                                       GMA=TWONRM/PRTN  SAME
4.                                       G=R-GMA*G        SAME
5.  W=C**(T-1)*G       SAME              SAME             SAME
6.  CG=C*W             SAME              SAME             SAME
7.  V=SA*M#*A*M*CG     SAME              SAME             SAME
8.  V=V+SC*W           SAME              SAME             SAME
9.  W=B**L*CG          SAME              SAME             SAME
10. V=V+SB*W           SAME              SAME             SAME
11.                    V=C**T*V                           SAME
12.                    GTQG=V*G                           SAME
13. GTQG=V*CG                            SAME
           TWONRM
14. DELTA= ------      SAME              SAME             SAME
            GTQG
                       NOTE:             NOTE:            NOTE:
                       TWONRM AND DELTA  ONLY CG IS       FOR CONSTANT
                       ARE NOT COMPUTED  COMPUTED FOR     DELTA, DELTA
                       FOR CONSTANT      CONSTANT         IS NOT
                       DELTA             DELTA            COMPUTED
*/
      
#include <cstdio>
#include <cmath>

#include "blkdta.h"
#include "geom.h"
#include "consts.h"
#include "uiod.h"
#include "blob.h"

#include "quad.h"

void quad_class::deltac(
  REAL* r,
  REAL* g,
  REAL* v,
  REAL* cg,
  REAL* w,
  REAL* delta,  // corr. hstau
  REAL* gma,    // corr. hstau
  INTEGER* list, 
  REAL* weight,
  BOOLEAN* alg,
  REAL* s
)
{
  static REAL twonrm; // set static. hstau
  static REAL prtn;// set static. hstau
  REAL gtqg;
  INTEGER i;

  INTEGER area;

  if(Blob.pix_basis) {
    area = GeoPar.area;
  }
  else {
    area = Blob.area;
  }
 
  if(iterx == 1) twonrm = 0.0; //corr. hstau

       
  if(!((bopt <= 2) && (popt <= 0))) {
    prtn = twonrm;

//C2.

    twonrm = 0.0;

    // MULTIFLY R BY S FOR SPLITTING

    if(gopt > 0) {
      for(i = 0; i < area; i++) {
        if(Blob.pix_basis || (!Blob.pix_basis && i%2 == Blob.pr)) {
          twonrm += s[i] * r[i] * r[i];
	}
      }
    }
    else {

      for(i = 0; i < area; i++) {
        if(Blob.pix_basis || (!Blob.pix_basis && i%2 == Blob.pr)) {
         twonrm += r[i] * r[i];
	}
      }
    }

    if(popt >= 1) {
      //fprintf(output, "\n\n      norm square of gradient vector is %15.6e", twonrm);
         fprintf(output, "\n\n      squared norm of the residual is %15.6e", twonrm);      //wei, bug 191, 12/2005   
    }

    if(twonrm <= Consts.zero) {
      //fprintf(output, "\n\n      norm of gradient vector is zero, iteration stops");
     fprintf(output, "\n\n       norm of the residual is zero, iteration stops");  //wei, bug 191, 12/2005  
      
      *alg = TRUE;
      return;
    }
  }

  // AT FIRST ITERATION FOR CONJUGATE GRADIANT SET G = R

  if(aopt > 2) {
    if(iterx == 1) {
      for(i = 0; i < area; i++) {
        g[i] = r[i];
      }
    }
//C3.
    else {
      if(bopt == 3) *gma = twonrm/prtn; //gma = twonrm/prtn;

//C4.

      for(i = 0; i < area; i++) {
        g[i] = r[i] + (*gma) * g[i];
	// g[i] = r[i] + gma * g[i];
      }
    }
  }
  // INITILIZE CG TO G

  if(gopt <= 0) {

    // NO SPLITTING REQUIRED

    for(i = 0; i < area; i++) {
      cg[i] = g[i];
    }
  }
  else {
    // SPLITTING

    for(i = 0; i < area; i++) {
      cg[i] = g[i] * s[i];
    }
  }

  if(t > 1) {
    if(copt != 5) {
      if(Blob.pix_basis) {
        adsmos(cg, t-1, cw1, cw2, cw3, cbcon);
      }
      else {
        badsmos(cg, t-1, cw1, cw2, cbcon);
      }
    }
    // NO NEED FOR W=C**(T-1)*G IF SC=0
  }

   
  if(sc > Consts.zero) { //

//C5.

    for(i = 0; i < area; i++) {
      w[i] = cg[i];
    }
  }
//C6.

  if(copt != 5) {
    if(Blob.pix_basis) {
      adsmos(cg,1, cw1, cw2, cw3, cbcon);
    }
    else {
      badsmos(cg,1, cw1, cw2, cbcon);
    }
  }
  // RETURN FOR CONSTANT DELTA AND AOPT=3

  if((aopt == 3) && (bopt <= 2)) return;

//C7.

  mtamx(cg, v, list, weight);

  if(sc > Consts.zero) {

//C8.

    for(i = 0; i < area; i++) {
      v[i] += sc * w[i];
    }
  }
//C9.


  if(sb > Consts.zero) {
    if(l > 0) {
      for(i = 0; i < area; i++) {
        w[i] = cg[i];
      }

      if(Blob.pix_basis) {
        adsmos(w, l, bw1, bw2, bw3, bbcon);
      }
      else {
        badsmos(w, l, bw1, bw2, bbcon);
      }
      
//C10.

      for(i = 0; i < area; i++) {
        v[i] += sb * w[i];
      }
    }
    else {
      for(i = 0; i < area; i++) {
        v[i] += sb * cg[i];
      }
    }
  }

  // CHECK FOR METHOD

  if(!((aopt == 1) || (aopt == 3))) {

//C11.

    if(copt != 5) {
      if(Blob.pix_basis) {
        adsmos(v, t, cw1, cw2, cw3, cbcon);
      }
      else {
        badsmos(v, t, cw1, cw2, cbcon);
      }
    }
    // FOR CONSTANT DELTA RETURN

    if(bopt <= 2) return;

//C12.

    gtqg = 0.0;

    if(gopt <= 0) {

      // INDIRECT NO SPLITTING

      for(i = 0; i < area; i++) {
        if(Blob.pix_basis || (!Blob.pix_basis && i%2 == Blob.pr)) {
        gtqg += v[i] * g[i];
	}
      }

    }
    else {

      // INDIRECT WITH SPLITTING

      for(i = 0; i < area; i++) {
        if(Blob.pix_basis || (!Blob.pix_basis && i%2 == Blob.pr)) {
          gtqg += v[i] * g[i] * s[i];
	}
      }
    }
  }
  else {
//C13.

// DIRECT METHOD CG = C**(T/2)*S**2*G

    gtqg = 0.0;

    for(i = 0; i < area; i++) {
      if(Blob.pix_basis || (!Blob.pix_basis && i%2 == Blob.pr)) {
        gtqg += v[i] * cg[i];
      }
    }
  }
  // CHECK FOR ZERO DENOMINATOR


  if(fabs(gtqg) > Consts.zero) {

//C14.

    *delta = twonrm / gtqg;
  
    return;
  }
  fprintf(output, "\n***zero denominator when computing delta iteratively, program stops***"); //not selfexplained hstau
  *alg = TRUE;
  return;
}      
