/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/quad_mtamx.cpp $
 $LastChangedRevision: 80 $
 $Date: 2014-07-01 21:01:54 -0400 (Tue, 01 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 Y = SA*M#*A*M*X

 X AND Y MUST BE DISTINCT
 */

#include <cstdio>
#include <cmath>

#include "blkdta.h"
#include "geom.h"
#include "noise.h"
#include "wray.h"
#include "anglst.h"
#include "ray.h"
#include "errfac.h"
#include "consts.h"

#include "blob.h"
#include "quad.h"

void quad_class::mtamx(REAL* x, REAL* y, INTEGER* list, REAL* weight)
{
	INTEGER i;
	INTEGER np;
	INTEGER nr;
	INTEGER numb;
	REAL snorm;
	REAL val;
	INTEGER nb;
	INTEGER k;
	REAL truval;
	REAL pix4;
	REAL theta;
	REAL sinth;
	REAL costh;

	REAL rinc;
	REAL rinc2;

	BOOLEAN alg;

	REAL hex_voronoi;
	INTEGER area;

	if (Blob.pix_basis)
	{
		area = GeoPar.area;
	}
	else
	{
		area = Blob.area;
	}

	hex_voronoi = Consts.sqrt3 / (REAL) 2.0 * Blob.delta * Blob.delta;

	// ZERO OUT Y

	for (i = 0; i < area; i++)
	{
		y[i] = 0.0;
	}

	// FOR BETTER PERFORMANCE LINE AND STRIP MODES ARE SEPERATED

	if (!GeoPar.strip)
	{

		// LINE MODE

		for (np = 0; np < GeoPar.prjnum; np++)
		{
			for (nr = 0; nr < GeoPar.nrays; nr++)
			{
				if (Blob.pix_basis)
				{
					wray(np, nr, list, weight, &numb, &snorm);
				}
				else
				{
					Blob.bwray(np, nr, list, weight, &numb, &snorm);
				}
				if (numb > 0)
				{
					val = 0.0;


					for (nb = 0; nb < numb; nb++)
					{
						k = list[nb];
						val += x[k] * weight[nb];
					}


					if ((eopt - 2) >= 0)
					{
						if ((eopt - 2) > 0)
						{
							truval = Anglst.prdta(np, nr);
							error_1 = uerror(np, nr, truval, &alg);
						}
						else
						{
							truval = Anglst.prdta(np, nr);
							error_1 = errfac(truval, 0.0, 0.0);
						}
					}
					val *= sa / error_1;


					for (nb = 0; nb < numb; nb++)
					{
						k = list[nb];
						y[k] += val * weight[nb];
					}
					// NEXT NR
				}
			}
			// NEXT NP

		}
		return;

		// STRIP MODE
	}

	rinc = GeoPar.pinc;
	if (Blob.pix_basis)
	{
		pix4 = sa * GeoPar.pixsiz * GeoPar.pixsiz * GeoPar.pixsiz
				* GeoPar.pixsiz;
	}
	else
	{
		pix4 = sa * hex_voronoi * hex_voronoi;
	}

	for (np = 0; np < GeoPar.prjnum; np++)
	{
		Anglst.getang(np, &theta, &sinth, &costh);
		if ((NoisePar.quanin > 0) && GeoPar.vri)
		{
			rinc = GeoPar.pinc * MAX0((REAL) fabs(costh), (REAL) fabs(sinth));
		}

		rinc2 = rinc * rinc;

		for (nr = 0; nr < GeoPar.nrays; nr++)
		{
			if (Blob.pix_basis)
			{
				ray(np, nr, list, weight, &numb, &snorm);
			}
			else
			{
				Blob.bray(np, nr, list, weight, &numb, &snorm);
			}

			if (numb > 0)
			{
				val = 0.0;

				for (nb = 0; nb < numb; nb++)
				{
					k = list[nb];
					val += x[k];
				}


				if ((eopt - 2) >= 0)
				{
					if ((eopt - 2) > 0)
					{
						truval = Anglst.prdta(np, nr);
						error_1 = uerror(np, nr, truval, &alg);
						goto L16;
					}
					else
					{
						truval = Anglst.prdta(np, nr);
						error_1 = errfac(truval, rinc, rinc2);
					}
				}

				L16: val *= pix4 / error_1;


				for (nb = 0; nb < numb; nb++)
				{
					k = list[nb];
					y[k] += val;
				}
				// NEXT NR
			}
		}
		// NEXT NP
	}
	return;
}
