/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/quad_resedu.cpp $
 $LastChangedRevision: 89 $
 $Date: 2014-07-02 17:24:53 -0400 (Wed, 02 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 X IS THE RECONSTRUCTED PICTURE

 (A)  R = SA*C**T*M#*A*(M*X-P)+(SB*C**T*B**L+SC*C**(T-1))*(X-XBAR)

 W IS TEMPORARY STORAGE VECTOR

 IF T IS ODD OR DELTA CONSTANT WITH AOPT=1 THEN:
 G = C**T*R    (G=R)


 OR
 (B)  R = M#*A*(M*X-P) AND FOR CONSTANT DELTA AND AOPT=1
 G = D**2*R    (G=R)

 (A)                          (B)

 1.       R=SA*M#*A*(M*X-P)            SAME
 2.                                    R=D**2*R
 3.       W=X-XBAR
 4.       X=B**L*W
 5.       R=R+SB*W
 6.       R=C**T*R
 7.       W=X-XBAR
 8.       W=C**(T-1)*W
 9.       R=R+SC*W
 */

#include <cstdio>

#include "blkdta.h"
#include "geom.h"
#include "consts.h"
#include "blob.h"

#include "quad.h"

void quad_class::resedu(REAL* x, REAL* r, REAL* w, REAL* d, INTEGER* list,
		REAL* weight, REAL* f, REAL* s)
{
	INTEGER i;
	INTEGER area;

	if (Blob.pix_basis)
	{
		area = GeoPar.area;
	}
	else
	{
		area = Blob.area;
	}

	if (sa > Consts.zero)
	{

//C1.
		mtamxp(x, r, list, weight);

		if (copt <= 3)
		{

			// IF DELTA IS NOT CONSTAT OR AOPT NOT 1 RETURN

			if ((bopt > 2) || (aopt != 1))
			{
				return;
			}
//C2.

			for (i = 0; i < area; i++)
			{
				r[i] *= d[i];
			}
			return;
		}
	}

	if (sb > Consts.zero)
	{

//C3.

		if (fopt > 2)
		{

			for (i = 0; i < area; i++)
			{
				w[i] = x[i] - f[i];
			}
		}
		else
		{
			for (i = 0; i < area; i++)
			{
				w[i] = x[i] - xbar;
			}
		}

		if (l != 0)
		{

//C4.

			if (Blob.pix_basis)
			{
				adsmos(w, l, bw1, bw2, bw3, bbcon);
			}
			else
			{
				badsmos(w, l, bw1, bw2, bbcon);
			}
//C5.
		}
		for (i = 0; i < area; i++)
		{
			r[i] += sb * w[i];
		}
	}
//C6.

	if (copt != 5)
	{
		if (Blob.pix_basis)
		{
			adsmos(r, t, cw1, cw2, cw3, cbcon);
		}
		else
		{
			badsmos(r, t, cw1, cw2, cbcon);
		}
	}

	if (sc > Consts.zero)
	{

//C7.

		if (fopt > 2)
		{

			for (i = 0; i < area; i++)
			{
				w[i] = x[i] - f[i];
			}
		}
		else
		{

			for (i = 0; i < area; i++)
			{
				w[i] = x[i] - xbar;
			}
		}

//C8.

		if ((t - 1 > 0) && (copt != 5))
		{
			if (Blob.pix_basis)
			{
				adsmos(w, t - 1, cw1, cw2, cw3, cbcon);
			}
			else
			{
				badsmos(w, t - 1, cw1, cw2, cbcon);
			}
		}

//C9.

		for (i = 0; i < area; i++)
		{
			r[i] += sc * w[i];
		}
	}

// FOR ONE STEP METHOD WITH DIRECT COMPUTATION OF RESIDUE
// WHICH DO NOT USE DELTAC OR DELTAD COMPUTE THE CORRECTING VECTOR
// MODIFY BY THE SPLITTING

	if ((aopt != 1) || (bopt >= 3) || (gopt == 0))
	{
		return;
	}
	for (i = 0; i < area; i++)
	{
		r[i] *= s[i];
	}

	if (copt != 5)
	{
		if (Blob.pix_basis)
		{
			adsmos(r, t, cw1, cw2, cw3, cbcon);
		}
		else
		{
			badsmos(r, t, cw1, cw2, cbcon);
		}
	}
	return;
}
