/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/read_eval_recon2.c $
 $LastChangedRevision: 92 $
 $Date: 2014-07-02 17:46:48 -0400 (Wed, 02 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 Previously part of SuperSNARK
 */

#include <string.h>
#include <strings.h>

#include "experimenter.h"
#include "errorc.h"
#include "read_eval_recon2.h"

/* ----------------------------- read_eval_recon2.c ----------------------

 This function reads from the eval file the normalized root mean
 square distance between a reconstruction and the phantom

 INPUTS:
 mode - number of MODEs read in so far by analyze.c.
 iteration - iteration number
 keywrd - keyword which defines the algorithm whose reconstruction is
 to be read in.
 

 OUTPUT:
 recon - double precision variable which stores the normalized root mean
 square distance between the (clipped) reconstruction and the
 (clipped) phantom.
 */

void read_eval_recon2(int mode, int iteration, char* keywrd, double* recon)
{
	int area, global = 0, iter;
	double average, distance;
	char string[MAXLINESIZE], test[MAXLINESIZE];
	FILE *evalfile;

	if ((evalfile = fopen("eval", "r")) == NULL)
		errorc("in read_eval_recon2.c: error in opening file ", "eval");

	/* read eval until the (mode+1)th occurrence of 'global' is reached. */
	while (global != (mode + 1))
	{

		fgets(string, sizeof(string), evalfile);
		if (strstr(string, "global") != NULL)
		{
			global++;
		}
		if (feof(evalfile))
		{
			errorc( "Error in read_eval_recon2.c: number of MODEs in comparison file exceeds number of globals in file", " eval");
		}
	}

	while (1)
	{
		/* Read eval file until a line which contains the string
		 'execution name' is reached. If this line contains the
		 keyword, break out of the while loop. If end of file is
		 reached then keyword was not found so terminate program. */

		fgets(string, sizeof(string), evalfile);

		if (strstr(string, "execution name") != NULL)
		{

			sscanf(string, "%*s%*s %s", test);
			if (strncasecmp(test, keywrd, 4) == 0)
			{
				break;
			}
		}

		if (feof(evalfile))
		{
			sprintf(string,
					"Error in read_eval_recon2.c: keyword %s not found in file",
					keywrd);
			errorc(string, " eval");
		}
	}

	while (1)
	{
		/* Continue reading eval file until a line of numbers is
		 reached then break out of while loop. If end of file
		 is encountered then terminate program. */

		fgets(string, sizeof(string), evalfile);
		if (sscanf(string, "%d %d %lf %lf", &iter, &area, &average, &distance) == 4)
		{
			break;
		}
		if (feof(evalfile))
		{
			errorc("Error in read_eval_recon2.c: end of file reached in file ", "eval");
		}
	}

	/* If necessary search the remaining lines of numbers until
	 the appropriate iteration number is found. Then save the
	 distance and break out of while loop. If approriate
	 iteration number is not found then terminate program. */

	if (iteration != 0)
	{

		/* if this first line of numbers has correct iteration
		 number save the distance and return*/

		if (iter == iteration)
		{
			*recon = distance;
			fclose(evalfile);
			return;
		}

		while (1)
		{

			fgets(string, sizeof(string), evalfile);
			sscanf(string, "%d %d %lf %lf", &iter, &area, &average, &distance);
			if (iter == iteration)
			{
				*recon = distance;
				break;
			}
			if (feof(evalfile))
			{
				sprintf(string, "Error in read_eval_recon2.c: %s --> iteration %d not found in file", keywrd, iteration);
				errorc(string, " eval");
			}
		}
	}

	// special case iteration == 0
	// the last iteration should be processed instead of a literal interpretation of 0
	// olangthaler, 03/14/14
	else
	{
		char prev[MAXLINESIZE];

		while (1)
		{
			strcpy(prev, string);
			fgets(string, sizeof(string), evalfile);
			if (strlen(string) < 3)
			{
				break;
			}
			if (feof(evalfile))
			{
				sprintf(string, "Error in read_eval_recon2.c: %s --> iteration %d not found in file", keywrd, iteration);
				errorc(string, " eval");
			}
		}

		sscanf(prev, "%d %d %lf %lf", &iter, &area, &average, &distance);
		*recon = distance;
	}

	fclose(evalfile); /* close eval file */
}
