/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/sart.cpp $
 $LastChangedRevision: 95 $
 $Date: 2014-07-02 19:43:36 -0400 (Wed, 02 Jul 2014) $
 $Author: agulati $
 ***********************************************************
 */

#include <cstdio>
#include <iostream>
#include "sart.h"
#include "anglst.h"
#include "pseudo.h"
#include "GeometricBehaviour.h"
#include "SARTConfig.h"

using namespace std;

sart::~sart() {
	this->Reset();
}

GeometricBehaviour* sart::geometricBehaviour = NULL;
REAL sart::relaxationParameter = 1.0;
REAL sart::zeroThreshold = 0.0;

INTEGER sart::Init() {
	this->geometricBehaviour = GeometricBehaviour::getInstance();
	return 0;
}

INTEGER sart::Reset() {
	if (this->geometricBehaviour != NULL) {
		delete this->geometricBehaviour;
		this->geometricBehaviour = NULL;
	}

	this->relaxationParameter = 1.0;
	this->zeroThreshold = 0.0;

	return 0;
}

BOOLEAN sart::Run(REAL* recon, INTEGER* list, REAL* weight, INTEGER iter) {

	// read configuration on first iteration
	if (iter == 1) {
		SARTConfig::readSARTConfiguration();
		this->relaxationParameter = SARTConfig::relaxiation;
		this->zeroThreshold = SARTConfig::zeroThreshold;
	}

	INTEGER numberOfPixels;
	REAL snorm = 0;

	REAL raySumMeasured;

	REAL currentRaySum;

	REAL rayWeightSum;
	REAL* pixelRayWeight = new REAL[this->geometricBehaviour->getArea()];
	REAL* pixelRayWeightSum = new REAL[this->geometricBehaviour->getArea()];

	REAL* correction = new REAL[this->geometricBehaviour->getArea()];

	// initialize values which are calculated once per iteration
	for (int i = 0; i < this->geometricBehaviour->getArea(); i++) {
		pixelRayWeight[i] = 0.0;
		pixelRayWeightSum[i] = 0.0;
		correction[i] = 0.0;
	}

	// loop through all rays
	for (int projectionNr = 0; projectionNr < GeoPar.prjnum; projectionNr++) {
		for (int rayNr = 0; rayNr < GeoPar.nrays; rayNr++) {

			raySumMeasured = Anglst.prdta(projectionNr, rayNr);

			this->geometricBehaviour->getRayTrace(projectionNr, rayNr, list,
					weight, &numberOfPixels, &snorm);

			// calculate current raysum
			currentRaySum = 0.0;
			rayWeightSum = 0.0;

			for (int i = 0; i < numberOfPixels; i++) {
				currentRaySum += recon[list[i]] * weight[i];
				rayWeightSum += weight[i];
				pixelRayWeight[list[i]] = weight[i];
				pixelRayWeightSum[list[i]] += weight[i];
			}

			if (rayWeightSum < this->zeroThreshold) {
				continue;
			}

			for (int i = 0; i < numberOfPixels; i++) {
				correction[list[i]] += pixelRayWeight[list[i]]
						* (raySumMeasured - currentRaySum) / rayWeightSum;
				pixelRayWeight[list[i]] = 0.0;
			}
		}
	}

	// update pixel
	for (int i = 0; i < this->geometricBehaviour->getArea(); i++) {

		if (pixelRayWeightSum[i] < this->zeroThreshold) {
			continue;
		}

		REAL oldRecon = recon[i];
		recon[i] += this->relaxationParameter / pixelRayWeightSum[i]
				* correction[i];
		if (recon[i] < 0.0) {
			recon[i] = 0.0;
		}
	}

	delete[] pixelRayWeight;
	delete[] pixelRayWeightSum;
	delete[] correction;

	return false;
}
