/*
 ***********************************************************
 $SNARK_Header: S N A R K  1 4 - A PICTURE RECONSTRUCTION PROGRAM $
 $HeadURL: svn://dig.cs.gc.cuny.edu/snark/trunk/src/snark/smooth.cpp $
 $LastChangedRevision: 97 $
 $Date: 2014-07-02 20:07:41 -0400 (Wed, 02 Jul 2014) $
 $Author: agulati $
 ***********************************************************

 A NINE-POINT WEIGHTED, NON-LINEAR SMOOTHING PROCEDURE.
 FOR DETAILS, SEE THE SNARK MANUAL.
 */

#include <cstdio>
#include <cmath>

#include "blkdta.h"
#include "consts.h"
#include "uiod.h"

#include "smooth.h"

void smooth(
REAL* a, // image
		INTEGER m, // number of rows
		INTEGER n, // number of columns
		REAL thresh, // treshold
		REAL w1, // weight of center
		REAL w2, // weight of horizontal and vertical neighbous
		REAL w3 // weight of diagonal neighbous
		)
{
	REAL* row1 = new REAL[n]; // row buffer 1
	REAL* row2 = new REAL[n]; // row buffer 2
	REAL* pres = row2; // current row before smooth
	REAL* prev = row1; // previous row before smooth

	BOOLEAN once = TRUE; // message flag

	INTEGER l = 0; // curent pixel index
	REAL centre; // curent pixel value

	REAL smuth; // weighted sum of neighbours
	REAL divide; // sum of weights
	REAL value; // neighbour value

	for (int i = 0; i < m; i++)
	{ // for every row

		REAL* itemp = prev; // swap the buffers
		prev = pres;
		pres = itemp;

		for (int j = 0; j < n; j++)
		{ // for every column

			centre = a[l];
			pres[j] = centre;
			smuth = w1 * centre;
			divide = w1;

			if (i != 0)
			{ // if not first row
				if (j != 0)
				{ // if not first column
					value = prev[j - 1]; // prev row prev column

					if ((REAL) fabs(value - centre) <= thresh)
					{
						smuth += w3 * value;
						divide += w3;
					}
				}

				value = prev[j]; // prev row same column

				if ((REAL) fabs(value - centre) <= thresh)
				{
					smuth += w2 * value;
					divide += w2;
				}

				if (j != (n - 1))
				{ // if not last column
					value = prev[j + 1]; // prev row next column

					if ((REAL) fabs(value - centre) <= thresh)
					{
						smuth += w3 * value;
						divide += w3;
					}
				}
			}

			if (j != 0)
			{ // if not first column
				value = pres[j - 1]; // same row prev column

				if ((REAL) fabs(value - centre) <= thresh)
				{
					smuth += w2 * value;
					divide += w2;
				}
			}

			if (j != (n - 1))
			{ // if not last column
				value = a[l + 1]; // same row next column

				if ((REAL) fabs(value - centre) <= thresh)
				{
					smuth += w2 * value;
					divide += w2;
				}
			}

			if (i != (m - 1))
			{ // if not last row
				if (j != 0)
				{ // if not first column
					value = a[l + n - 1]; // next row prev column

					if ((REAL) fabs(value - centre) <= thresh)
					{
						smuth += w3 * value;
						divide += w3;
					}
				}

				value = a[l + n]; // next row same column

				if ((REAL) fabs(value - centre) <= thresh)
				{
					smuth += w2 * value;
					divide += w2;
				}

				if (j != (n - 1))
				{ // if not last column
					value = a[l + n + 1]; // next row next column

					if ((REAL) fabs(value - centre) <= thresh)
					{
						smuth += w3 * value;
						divide += w3;
					}
				}
			}

			if ((REAL) fabs(divide) > Consts.zero)
			{
				a[l] = smuth / divide;
			}
			else
			{
				if (once)
				{
					fprintf(output, "\n **** warning - sum of weights = zero");
					once = FALSE;
				}

//pragma message( "to check!" )
				a[l] = ((smuth - Consts.infin) > 0) ? (REAL) 1.0 : (smuth == Consts.infin) ? (REAL) 0.0 : (REAL) -1.0;
			}

			l++;
		}
	}

	delete[] row1; // bug 92 - Lajos - 03/02/2005
	delete[] row2; // bug 92 - Lajos - 03/02/2005

	return;
}
