/** @file chooseYVar.cpp
    @package snark14Display
    @author Bruno M. Carvalho and Deniz Sarioz
    licensed under (open-source) QPL v1.0
    which accompanies this distribution in the file QPL
 */

#include "chooseYVar.hpp"
#include "SnarkDisplay.hpp"
#include "displaylines.hpp"
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define EPSILON 0.00000001

/** 
 *  Constructs a chooseYVarWindow which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'fl', initial topleft position to (x,y)
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
chooseYVarWindow::chooseYVarWindow(QWidget* parent, const char* name, bool modal, WFlags fl, int x, int y)
: QDialog(parent, name, modal, fl) {
    QPoint topleft;
        static const int WINWIDTH = 300;
        static const int WINHEIGHT = 280;
    if (!name)
        setName("selectEvalExecutionsWindow");
        resize( WINWIDTH, WINHEIGHT );
    // setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, sizePolicy().hasHeightForWidth() ) );
        setMinimumSize( QSize( WINWIDTH, WINHEIGHT ) );
//        setMaximumSize( QSize( WINWIDTH, WINHEIGHT ) );
    setCaption(tr("Select Y Var to plot from selected Evaluate Executions"));
    topleft.setX(x);
    topleft.setY(y);
    this->move(topleft);

    QBoxLayout *topLayout = new QVBoxLayout(this);
//    topLayout.setDirection(QBoxLayout::TopToBottom);
    QBoxLayout *yaxisbuttonLayout = new QVBoxLayout(topLayout);
    QBoxLayout *colorbuttonLayout = new QVBoxLayout(topLayout);

    // ain't broke: don't fix it.
    yaxisbuttongroup = new QButtonGroup(this, "yaxisbuttongroup");
    yaxisbuttongroup->setTitle(tr("Y Axis"));
    yaxisbuttongroup->setAlignment(int( QButtonGroup::AlignHCenter));
    globalbutton = new QRadioButton(yaxisbuttongroup, "globalbutton");
    globalbutton->setGeometry(QRect(10, 20, 140, 30));
    globalbutton->setText(tr("Global"));
    globalbutton->setChecked(false);
    if (!myEES.selectionHasGlobal())
        globalbutton->setEnabled(false); // can't click it
    QObject::connect(globalbutton, SIGNAL(clicked()), this, SLOT(activateGlobalErrors()));

    globalerrors = new QComboBox(yaxisbuttongroup, "globalerrors");
    globalerrors->setGeometry(QRect(150, 20, 140, 30));
    globalerrors->setEnabled(false);
    // it seems a bit idiosyncratic to pass int as a parameter, but it seems the only way
    QObject::connect(globalerrors, SIGNAL(activated(int)), this, SLOT(maybeGetCalculatedExtrema()));

    pointbutton = new QRadioButton(yaxisbuttongroup, "pointbutton");
    pointbutton->setGeometry(QRect(10, 50, 140, 30));
    pointbutton->setText(tr("Point-by-point"));
    pointbutton->setChecked(false);
    if (!myEES.selectionHasPBP()) {
        pointbutton->setEnabled(false); // can't click it
    }
    QObject::connect(pointbutton, SIGNAL(clicked()), this, SLOT(activatePointErrors()));
    pointerrors = new QComboBox(yaxisbuttongroup, "pointerrors");
    pointerrors->setGeometry(QRect(150, 55, 140, 30));
    pointerrors->setEnabled(false);
    QObject::connect(pointerrors, SIGNAL(activated(int)), this, SLOT(maybeGetCalculatedExtrema()));

    yaxisbuttonLayout->addWidget(yaxisbuttongroup);


    // jk 1/13/2009 adding grayScale option for graphs
    // jklukowska 6/7/2009 modified to be radio button with option of either
    // color or grayscale
    colorbuttongroup = new QButtonGroup(this, "colorbuttongroup");
    colorbuttongroup->setTitle(tr("Draw graphs in "));
    colorbuttongroup->setAlignment(int( QButtonGroup::AlignHCenter));
    grayScaleButton = new QRadioButton(colorbuttongroup, "grayScaleButton");
    grayScaleButton->setGeometry(QRect(10, 15, 130, 20));
    grayScaleButton->setText(tr(" grayscale"));
    grayScaleButton->setEnabled(true);
    grayScaleButton->setChecked(false);
    colorButton = new QRadioButton(colorbuttongroup, "colorButton");
    colorButton->setGeometry(QRect(160, 15, 130, 20));
    colorButton->setText(tr(" color"));
    colorButton->setEnabled(true);
    colorButton->setChecked(true);
    colorbuttonLayout->addWidget(colorbuttongroup);

    QString* sameLabel = new QString("Use same extrema for all evaluate executions");
    sameExtremaForAllCheckBox = new QCheckBox(*sameLabel, this, "sameExtrema");
    sameExtremaForAllCheckBox->setTristate(false); // don't want a tristate button
    sameExtremaForAllCheckBox->setEnabled(true);
    sameExtremaForAllCheckBox->setChecked(true);
    topLayout->addWidget(sameExtremaForAllCheckBox);
    QObject::connect(sameExtremaForAllCheckBox, SIGNAL(clicked()), this, SLOT(sameExtremaClicked()));

    minx = new QLineEdit(this, "minx");
    maxx = new QLineEdit(this, "maxx");
    miny = new QLineEdit(this, "miny");
    maxy = new QLineEdit(this, "maxy");

    deactivateExtremaFields();

    QBoxLayout* x_extrema_layout = new QHBoxLayout(topLayout);
    minxlabel = new QLabel(this, "minxlabel");
    minxlabel->setText(tr("  Min Iter:"));
    maxxlabel = new QLabel(this, "maxxlabel");
    maxxlabel->setText(tr("  Max Iter:"));
    x_extrema_layout->addWidget(minxlabel);
    x_extrema_layout->addWidget(minx);
    x_extrema_layout->addWidget(maxxlabel);
    x_extrema_layout->addWidget(maxx);

    QBoxLayout* y_extrema_layout = new QHBoxLayout(topLayout);
    minylabel = new QLabel(this, "minxlabel");
    minylabel->setText(tr("  Min Y:"));
    maxylabel = new QLabel(this, "maxxlabel");
    maxylabel->setText(tr("  Max Y:"));
    y_extrema_layout->addWidget(minylabel);
    y_extrema_layout->addWidget(miny);
    y_extrema_layout->addWidget(maxylabel);
    y_extrema_layout->addWidget(maxy);


    proceedbutton = new QPushButton(this, "proceedbutton");
    proceedbutton->setText(tr("Proceed"));
    proceedbutton->setEnabled(false);
    QObject::connect(proceedbutton, SIGNAL(clicked()), this, SLOT(proceed()));
    topLayout->addWidget(proceedbutton);

    closebutton = new QPushButton(this, "closebutton");
    closebutton->setText(tr("Close Window"));
    QObject::connect(closebutton, SIGNAL(clicked()), this, SLOT(closeWindow()));

    topLayout->addWidget(closebutton);
    topLayout->activate();
}

/**  
 *  Destroys the object and frees any allocated resources
 */
chooseYVarWindow::~chooseYVarWindow() {
    // no need to delete child widgets, Qt does it all for us
}

void chooseYVarWindow::closeWindow() {
    //  accept();
    reject();
}

void chooseYVarWindow::closeEvent(QCloseEvent * e) {
    e->accept();
}

/** 
@param void
@author Bruno M. Carvalho , Deniz Sarioz
@version 1.1 */
void chooseYVarWindow::activateGlobalErrors() {
    pointerrors->setEnabled(false);
    if (myEES.selectionHasGlobal()) {
        proceedbutton->setEnabled(true);
        globalerrors->clear();
        globalerrors->insertItem("Average", -1);
        globalerrors->insertItem("Distance", -1);
        globalerrors->insertItem("Relative Error", -1);
        globalerrors->insertItem("Variance", -1);
        globalerrors->insertItem("Std. Deviation", -1);
        if (myEES.selectionHasResidual()) {
            globalerrors->insertItem("Residual", -1);
        }
        if (myEES.selectionHasKullback()) {
            globalerrors->insertItem("KL Distance", -1);
        }
        globalerrors->setEnabled(true);
        maybeGetCalculatedExtrema();
    }
}

/** 
@param void
@author Bruno M. Carvalho , Deniz Sarioz
@version 1.1 */
void chooseYVarWindow::activatePointErrors() {
    globalerrors->setEnabled(false);
    if (myEES.selectionHasPBP()) {
        proceedbutton->setEnabled(true);
        pointerrors->clear();
        for (unsigned idx = 0; idx < myEES.get_num_Es(); idx++) {
            QString elabel(tr("e(%1)").arg(idx));
            // amazingly enough, this is parsed correctly.
            pointerrors->insertItem(elabel, -1);
        }
        pointerrors->setEnabled(true);
        maybeGetCalculatedExtrema();
    }
}

void chooseYVarWindow::maybeGetCalculatedExtrema() {
    if (sameExtremaForAllCheckBox->isChecked())
        getCalculatedExtrema();
}

//  int chooseYVarWindow::getYAxis() { return 1; }
//  int chooseYVarWindow::getMinX() { return 1; }
//  int chooseYVarWindow::getMaxX() { return 1; }
//  double chooseYVarWindow::getMinY() { return 1.0; }
//  double chooseYVarWindow::getMaxY() { return 1.0; }

void chooseYVarWindow::sameExtremaClicked() {
    if (sameExtremaForAllCheckBox->isChecked()) {
        //   std::cout << "sameExtrema CheckBox is checked." << std::endl;
        getCalculatedExtrema();
    } else {
        deactivateExtremaFields();
        //  std::cout << "sameExtrema CheckBox is unchecked." << std::endl;
    }
}

void chooseYVarWindow::activateExtremaFields() {
    minx->setEnabled(true);
    maxx->setEnabled(true);
    miny->setEnabled(true);
    maxy->setEnabled(true);
}

void chooseYVarWindow::deactivateExtremaFields() {
    minx->clear();
    maxx->clear();
    miny->clear();
    maxy->clear();
    minx->setEnabled(false);
    maxx->setEnabled(false);
    miny->setEnabled(false);
    maxy->setEnabled(false);
}

/** getCalculatedExtrema fetches from myEES the extrema of iteration and y-var
    over entire evaluate executions selection
    also reads in selection...
 */
void chooseYVarWindow::getCalculatedExtrema() {
    if (globalerrors->isEnabled()) {
        /// std::cout << "getting Calculated Extrema for some global Yvar" << std::endl;
        activateExtremaFields();
        minx->setText(tr("%1") . arg(myEES.getMinIterViaGlobal()));
        maxx->setText(tr("%1") . arg(myEES.getMaxIterViaGlobal()));
        int idx = globalerrors->currentItem();
        this->globY = static_cast<globalYVar_t> (idx);
        miny->setText(tr("%1") . arg(myEES.getMinYGlobal(this->globY)));
        maxy->setText(tr("%1") . arg(myEES.getMaxYGlobal(this->globY)));
    } else if (pointerrors->isEnabled()) {
        /// std::cout << "getting Calculated Extrema for some PBP Yvar" << std::endl;
        activateExtremaFields();
        minx->setText(tr("%1") . arg(myEES.getMinIterViaPBP()));
        maxx->setText(tr("%1") . arg(myEES.getMaxIterViaPBP()));
        int idx = pointerrors->currentItem();
        // a minor hack
        this->PBPY = static_cast<unsigned> (idx);
        miny->setText(tr("%1") . arg(myEES.getMinYPBP(this->PBPY)));
        maxy->setText(tr("%1") . arg(myEES.getMaxYPBP(this->PBPY)));
    } else {
        //  std::cout << "need to check either global or PBP" << std::endl;
    }
}

/** Called when proceedbutton "Proceed" is clicked:
    based on the sameExtremaForAllCheckBox state,
    either spawns plots or new dialog boxes.
    Does some checks before sending params to plots (no garbage out).
@param void
@author Bruno M. Carvalho
@version 1.0 */
void chooseYVarWindow::proceed() {
    //  std::cout << "in chooseYVarWindow::proceed()" << std::endl;
    bool complain = false;
    if (sameExtremaForAllCheckBox->isChecked()) {
        // attempt to call plot windows after error checking
        bool ok = true;
        int minXVal = minx->text().toInt(&ok);
        if ((!ok) || (minXVal < 0)) complain = true; // allow minXVal to be 0
        int maxXVal = maxx->text().toInt(&ok);
        if (!ok) complain = true;
        if (maxXVal <= 0) complain = true;
        if (minXVal >= maxXVal) complain = true;
        double minYVal = miny->text().toDouble(&ok);
        if (!ok) complain = true;
        double maxYVal = maxy->text().toDouble(&ok);
        if (!ok) complain = true;
        // allow negative values for minYVal, even though there are no negative ones in data
        if (maxYVal <= 0) complain = true;
        if (minYVal >= maxYVal) complain = true;
        if (complain) {
            std::cerr << "Invalid combination of X and Y parameters." << std::endl;
        } else { // passed error checks
            if (std::verbose >= 2) {
                std::cout << "Proceeding to plot with iter: [" << minXVal
                        << "," << maxXVal << "]; y: [" << minYVal << ","
                        << maxYVal << "], ";
            }
            if (pointerrors->isEnabled()) {
                if (std::verbose >= 2) {
                    std::cout << "YVar PBP e " << (this->PBPY) << std::endl;
                }
                spawnPlots(true, minXVal, maxXVal, minYVal, maxYVal);
            } else if (globalerrors->isEnabled()) {
                if (std::verbose >= 2) {
                    std::cout << "YVar global " << (this->globY) << std::endl;
                }
                spawnPlots(false, minXVal, maxXVal, minYVal, maxYVal);
            }
        }
    } else { // spawn individual dialog boxes for each evaluate execution
        if (std::verbose >= 3) {
            std::cout << "Spawning individual query boxes for parameters; ";
        }
        if (pointerrors->isEnabled()) {
            (this->PBPY) = static_cast<unsigned> (pointerrors->currentItem());
            if (std::verbose >= 2) {
                std::cout << "YVar is PBP e " << (this->PBPY) << std::endl;
            }
            spawnDialogs(true);
        } else if (globalerrors->isEnabled()) {
            (this->globY) = static_cast<globalYVar_t> (globalerrors->currentItem());
            if (std::verbose >= 2) {
                std::cout << "YVar is global " << (this->globY) << std::endl;
            }
            spawnDialogs(false);
        } else {
            std::cerr << "meaningless state in proceed()" << std::endl;
        }
    }
} // --chooseYVarWindow::proceed()

/** ask the EES to plot itself based on these params
 * ONLY to be called from proceed()
 */
void chooseYVarWindow::spawnPlots(bool plotPBP, int minXVal, int maxXVal, double minYVal, double maxYVal) {
    // but ONLY the selected evaluate executions'.
    // EES does the plotting including spawning of windows... *this* window becomes their parent.
    if (plotPBP) {
        myEES.spawnAllPlotsPBP(this, this->PBPY, minXVal, maxXVal, minYVal, maxYVal, grayScaleButton->isChecked());
    } else {
        myEES.spawnAllPlotsGlob(this, this->globY, minXVal, maxXVal, minYVal, maxYVal, grayScaleButton->isChecked());
        // displaylineswindow( QWidget* parent,  const char* name, QPixmap* pixmap, bool modal, WFlags fl , int 20, int 20 );
    }
} // --chooseYVarWindow::spawnPlots()

/** ask the EES to spawn plotting range dialogs based on YVar selection
 * ONLY to be called from proceed()
 */
void chooseYVarWindow::spawnDialogs(bool plotPBP) {
    if (std::verbose >= 2) {
        std::cout << "in chooseYVarWindow::spawnDialogs()" << std::endl;
    }
    if (plotPBP) {
        myEES.spawnDialogsPBP(this, this->PBPY);
    } else {
        myEES.spawnDialogsGlob(this, this->globY);
    }
} // --chooseYVarWindow::spawnDialogs()


/*
 Joanna Klukowska (added 2/2/2009)
*/
bool chooseYVarWindow::isGrayScaleChecked() {
    return (grayScaleButton->isChecked());
}
