/** @file selectEvalExecutions.cpp
    @package snark14Display
    @author Bruno M. Carvalho and Deniz Sarioz
    licensed under (open-source) QPL v1.0
    which accompanies this distribution in the file QPL
*/

#include "variables.hpp"
#include "selectEvalExecutions.hpp"
#include "chooseYVar.hpp"
#include "SnarkDisplay.hpp"

#include <stdio.h>
#include <qapplication.h>
#include <qframe.h>
#include <qlistbox.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>

// extern displaywindow **displaywindowset;

/* 
 *  Constructs a  selectEvalExecutionsWindow which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */

selectEvalExecutionsWindow::selectEvalExecutionsWindow( QWidget* parent,  const char* name, bool modal, WFlags fl , int x , int y )
  : QDialog( parent, name, modal, fl )
{
    QPoint topleft;

    ///    myYVarWinP = 0;

    const int WINWIDTH = 760;
    const int WINHEIGHT = 350;
    if ( !name )
	setName( "selectEvalExecutionsWindow" );
    resize( WINWIDTH, WINHEIGHT );     
    setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, sizePolicy().hasHeightForWidth() ) );
    setMinimumSize( QSize( WINWIDTH, WINHEIGHT ) );
    setMaximumSize( QSize( WINWIDTH, WINHEIGHT ) );
    setCaption( tr( "Select Evaluate Executions to be plotted" ) );

    ///    Parent=parent;
    topleft.setX(x);
    topleft.setY(y);
    this->move(topleft);

    const int BUTTONWIDTH = 140; // for (most) buttons
    const int BUTTONHEIGHT = 40;
    const int LRPADDING = 10; // left-to-right padding
    openbutton = new QPushButton( this, "openbutton" );
    openbutton->setGeometry( QRect( LRPADDING, 15, BUTTONWIDTH, BUTTONHEIGHT ) ); 
    openbutton->setText( tr( "Proceed" ) ); // makes more sense than "open"
    QObject::connect(openbutton,SIGNAL(clicked()),this,SLOT(openEvalExecutions()));

    selectbutton = new QPushButton( this, "selectbutton" );
    selectbutton->setGeometry( QRect( LRPADDING, 65, BUTTONWIDTH, BUTTONHEIGHT ) ); 
    selectbutton->setText( tr( "Select All" ) );
    QObject::connect(selectbutton,SIGNAL(clicked()),this,SLOT(selectAll()));

    clearselectbutton = new QPushButton( this, "clearselectbutton" );
    clearselectbutton->setGeometry( QRect( LRPADDING, 115, BUTTONWIDTH, BUTTONHEIGHT ) ); 
    clearselectbutton->setText( tr( "Clear Selected" ) );
    QObject::connect(clearselectbutton,SIGNAL(clicked()),this,SLOT(clearSelected()));

    evalExecutionsListBox = new QListBox( this, "evalExecutionsListBox" );
    evalExecutionsListBox->setSelectionMode(QListBox::Multi);
    QFont f = QFont("courier",10,25,false);
    f.setFixedPitch(true);
    evalExecutionsListBox->setFont(f);

    for(unsigned i=0; i < myEES.num_eval_execs(); i++) {
      ///      QString s = QString("Execution %1" ) . arg(i);
      QString s(static_cast<char*>(myEES.getExecName(i)));
      evalExecutionsListBox->insertItem(s);      
    }
    int tmpi = BUTTONWIDTH + LRPADDING*2;
    evalExecutionsListBox->setGeometry( QRect( tmpi, 15, WINWIDTH - tmpi - LRPADDING , 250 ) ); 
    
    Line6 = new QFrame( this, "Line6" );
    Line6->setGeometry( QRect( 0, 280, WINWIDTH, 16 ) ); 
    Line6->setFrameStyle( QFrame::HLine | QFrame::Sunken );

    closebutton = new QPushButton( this, "closebutton" );
    closebutton->setGeometry( QRect( 380, 300, 100, 40 ) ); 
    closebutton->setText( tr( "Close" ) );
    QObject::connect(closebutton,SIGNAL(clicked()),this,SLOT(reject()));
}

/*  
 *  Destroys the object and frees any allocated resources
 */
selectEvalExecutionsWindow::~selectEvalExecutionsWindow()
{
  // no need to delete child widgets, Qt does it all for us
}

void selectEvalExecutionsWindow::openEvalExecutions()
{
  if(std::verbose >= 2) std::cout << "in selectEvalExecutionsWindow::openEvalExecutions()" << std::endl;
  //  int c=0,x,y;
  int x,y;
  QPoint topleft;
  topleft=this->pos();
  x=topleft.x();
  y=topleft.y();
  for(unsigned i=0; i < myEES.num_eval_execs(); i++) {
    if(evalExecutionsListBox->isSelected(i)) {
      myEES.select_exec(i);
    } else {
      myEES.deselect_exec(i);
    }
  } // for
  ///// just to debug: 
  ///  myEES.show_selected();

  bool hasG, hasP;
  // if stmts have necessary side effects
  if(hasG = myEES.selectionHasGlobal()) {
    //    if(std::verbose >=1) std::cout << "The selection has global." << std::endl;
  }
  if(hasP = myEES.selectionHasPBP()) {
    //    if(std::verbose >=1) std::cout << "The selection has PBP." << std::endl;
  }
  if(!(hasG || hasP)) {
    std::cout << "Selection has neither global nor PBP!  Nothing to plot!" << std::endl;
  } else {
    // do something like open a brand new window... onward!
    QPoint topleft;
    topleft = this->pos();
    //    chooseYVarWindow* w;
    ///    if(myYVarWinP) delete myYVarWinP;
    //    myYVarWinP = new chooseYVarWindow(this, "blah", true, topleft.x(), topleft.y() );
    chooseYVarWindow* myYVarWinP;
    myYVarWinP = new chooseYVarWindow(desk, "Eval::chooseYVarWindow", false, 0, (this->pos()).x() + 30, (this->pos()).y() + 50 );
// w = new chooseYVarWindow(this, "Please choose the Y-variable to be plotted.", false, topleft.x() - 10, topleft.y() - 10);
    myYVarWinP->show();
// w->show();
  }
} // --selectEvalExecutionsWindow::openEvalExecutions()

void selectEvalExecutionsWindow::selectAll()
{
  evalExecutionsListBox->selectAll(true);
}

void selectEvalExecutionsWindow::clearSelected()
{
  evalExecutionsListBox->selectAll(false);
}

