/** @file execute.cpp
    @package snark05Input
    @author Bruno M. Carvalho and Deniz Sarioz
    licensed under (open-source) QPL v1.0
    which accompanies this distribution in the file QPL
*/

#include "execute.h"
#include "misc.h"
#include "variables.h"
#include <stdio.h>

#include <qbuttongroup.h>
#include <qframe.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qmultilineedit.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qmessagebox.h>

/** 
 *  Constructs a executewindow which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
@param void
@author Bruno M. Carvalho (modified a little by deniz)
@version 1.0 */
executewindow::executewindow( QWidget* parent,  const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )
{
    if ( !name )
	setName( "executewindow" );
    resize( 400, 600 ); 
    setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, sizePolicy().hasHeightForWidth() ) );
    setMinimumSize( QSize( 400, 700 ) );
    setMaximumSize( QSize( 400, 700 ) );
    setCaption( tr( "Execute Window" ) );

    executelabel = new QLabel( this, "executelabel" );
    executelabel->setGeometry( QRect( 10, 10, 120, 30 ) ); 
    executelabel->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, executelabel->sizePolicy().hasHeightForWidth() ) );
    executelabel->setMinimumSize( QSize( 120, 30 ) );
    executelabel->setMaximumSize( QSize( 120, 30 ) );
    executelabel->setText( tr( "EXECUTE" ) );

    executebuttongroup = new QButtonGroup( this, "executebuttongroup" );
    executebuttongroup->setGeometry( QRect( 40, 40, 160, 170 ) ); 
    executebuttongroup->setTitle( tr( "type" ) );
    executebuttongroup->setAlignment( int( QButtonGroup::AlignHCenter ) );

    zerobutton = new QRadioButton( executebuttongroup, "zerobutton" );
    zerobutton->setGeometry( QRect( 10, 20, 130, 24 ) ); 
    zerobutton->setText( tr( "ZERO" ) ); 

    averagebutton = new QRadioButton( executebuttongroup, "averagebutton" );
    averagebutton->setGeometry( QRect( 10, 50, 110, 24 ) ); 
    averagebutton->setText( tr( "AVERAGE" ) ); 

    continuebutton = new QRadioButton( executebuttongroup, "continuebutton" );
    continuebutton->setGeometry( QRect( 10, 80, 110, 24 ) ); 
    continuebutton->setText( tr( "CONTINUE" ) );

    // added by deniz:
    phantombutton = new QRadioButton( executebuttongroup, "phantombutton" );
    phantombutton->setGeometry( QRect( 10, 110, 110, 24 ) ); 
    phantombutton->setText( tr( "PHANTOM" ) );

    nonebutton = new QRadioButton( executebuttongroup, "nonebutton" );
    nonebutton->setGeometry( QRect( 10, 140, 80, 24 ) ); 
    nonebutton->setText( tr( "NONE" ) ); 
    nonebutton->setChecked(true);

    algname = new QLineEdit( this, "algname" );
    algname->setGeometry( QRect( 30, 230, 350, 28 ) );

    algnamelabel = new QLabel( this, "algnamelabel" );
    algnamelabel->setGeometry( QRect( 170, 210, 70, 20 ) ); 
    algnamelabel->setText( tr( "algname" ) );

    executebuttongroup2 = new QButtonGroup( this, "executebuttongroup2" );
    executebuttongroup2->setGeometry( QRect( 40, 270, 160, 100 ) ); 
    executebuttongroup2->setTitle( tr( "" ) );
    executebuttongroup2->setAlignment( int( QButtonGroup::AlignHCenter ) );

    contourbutton = new QRadioButton( executebuttongroup2, "contourbutton" );
    contourbutton->setGeometry( QRect( 10, 10, 110, 30 ) ); 
    contourbutton->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, contourbutton->sizePolicy().hasHeightForWidth() ) );
    contourbutton->setMinimumSize( QSize( 110, 30 ) );
    contourbutton->setMaximumSize( QSize( 110, 30 ) );
    contourbutton->setText( tr( "CONTOUR" ) );
    QObject::connect(contourbutton,SIGNAL(clicked()),this,SLOT(activateContourSmoothFields()));

    smoothbutton = new QRadioButton( executebuttongroup2, "smoothbutton" );
    smoothbutton->setGeometry( QRect( 10, 40, 110, 30 ) ); 
    smoothbutton->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, smoothbutton->sizePolicy().hasHeightForWidth() ) );
    smoothbutton->setMinimumSize( QSize( 110, 30 ) );
    smoothbutton->setMaximumSize( QSize( 110, 30 ) );
    smoothbutton->setText( tr( "SMOOTH" ) );
    QObject::connect(smoothbutton,SIGNAL(clicked()),this,SLOT(activateContourSmoothFields()));

    nonebutton2 = new QRadioButton( executebuttongroup2, "nonebutton2" );
    nonebutton2->setGeometry( QRect( 10, 70, 130, 24 ) ); 
    nonebutton2->setText( tr( "NONE" ) );
    nonebutton2->setChecked(true);
    QObject::connect(nonebutton2,SIGNAL(clicked()),this,SLOT(activateContourSmoothFields()));

    nameoftheexecution = new QLineEdit( this, "nameoftheexecution" );
    nameoftheexecution->setGeometry( QRect( 30, 390, 350, 28 ) ); 

    nameoftheexecutionlabel = new QLabel( this, "nameoftheexecutionlabel" );
    nameoftheexecutionlabel->setGeometry( QRect( 110, 370, 180, 20 ) ); 
    nameoftheexecutionlabel->setText( tr( "name_of_the_execution" ) );







    algvariables = new QMultiLineEdit( this, "algvariables" );
    algvariables->setGeometry( QRect( 30, 568, 350, 60 ) );

    algvariableslabel = new QLabel( this, "algvariableslabel" );
    algvariableslabel->setGeometry( QRect( 100, 535, 220, 20 ) ); 
    algvariableslabel->setText( tr( "algorithm specific variables" ) );







    threshold = new QLineEdit( this, "threshold" );
    threshold->setGeometry( QRect( 30, 442, 80, 28 ) ); 
    threshold->setEnabled(FALSE);

    thresholdlabel = new QLabel( this, "thresholdlabel" );
    thresholdlabel->setGeometry( QRect( 35, 422, 70, 20 ) ); 
    thresholdlabel->setText( tr( "threshold" ) ); 
    thresholdlabel->setEnabled(FALSE);

    weight1 = new QLineEdit( this, "weight1" );
    weight1->setGeometry( QRect( 120, 442, 80, 28 ) ); 
    weight1->setEnabled(FALSE);

    weight1label = new QLabel( this, "weight1label" );
    weight1label->setGeometry( QRect( 130, 422, 60, 20 ) ); 
    weight1label->setText( tr( "weight1" ) ); 
    weight1label->setEnabled(FALSE);

    weight2 = new QLineEdit( this, "weight2" );
    weight2->setGeometry( QRect( 210, 442, 80, 28 ) ); 
    weight2->setEnabled(FALSE);

    weight2label = new QLabel( this, "weight2label" );
    weight2label->setGeometry( QRect( 220, 422, 60, 20 ) ); 
    weight2label->setText( tr( "weight2" ) );
    weight2label->setEnabled(FALSE);

    weight3 = new QLineEdit( this, "weight3" );
    weight3->setGeometry( QRect( 300, 442, 80, 28 ) ); 
    weight3->setEnabled(FALSE);

    weight3label = new QLabel( this, "weight3label" );
    weight3label->setGeometry( QRect( 310, 422, 60, 20 ) ); 
    weight3label->setText( tr( "weight3" ) ); 
    weight3label->setEnabled(FALSE);

    iterationflaglabel = new QLabel( this, "iterationflaglinelabel" );
    iterationflaglabel->setGeometry( QRect( 140, 482, 150, 20 ) ); 
    iterationflaglabel->setText( tr( "iteration_flag_line" ) );
    iterationflaglabel->setEnabled(FALSE);

    iterationflagline = new QLineEdit( this, "iterationflagline" );
    iterationflagline->setGeometry( QRect( 30, 502, 350, 28 ) ); 
    iterationflagline->setEnabled(FALSE);

    Line5 = new QFrame( this, "Line5" );
    Line5->setGeometry( QRect( -15, 625, 450, 16 ) ); 
    Line5->setFrameStyle( QFrame::HLine | QFrame::Sunken );

    okbutton = new QPushButton( this, "okbutton" );
    okbutton->setGeometry( QRect( 50, 645, 100, 40 ) ); 
    okbutton->setText( tr( "OK" ) );
    QObject::connect(okbutton,SIGNAL(clicked()),this,SLOT(checkValues()));

    cancelbutton = new QPushButton( this, "cancelbutton" );
    cancelbutton->setGeometry( QRect( 250, 645, 100, 40 ) ); 
    cancelbutton->setText( tr( "Cancel" ) );
    QObject::connect(cancelbutton,SIGNAL(clicked()),this,SLOT(reject()));
}

/**  
 *  Destroys the object and frees any allocated resources
 */
executewindow::~executewindow()
{
    // no need to delete child widgets, Qt does it all for us
}


/** Activates type, algname and other fields 
@param void
@author Bruno M. Carvalho
@version 1.0 */
/* void executewindow::activateFields()
{
  if(executebutton->isChecked()) {
    executebuttongroup->setEnabled(TRUE);
    zerobutton->setEnabled(TRUE);
    averagebutton->setEnabled(TRUE);
    continuebutton->setEnabled(TRUE);
    nonebutton->setEnabled(TRUE);
    algname->setEnabled(TRUE);
    algnamelabel->setEnabled(TRUE);
    executebuttongroup2->setEnabled(TRUE);
    contourbutton->setEnabled(TRUE);
    smoothbutton->setEnabled(TRUE);
    nonebutton2->setEnabled(TRUE);
    nameoftheexecution->setEnabled(TRUE);
    nameoftheexecutionlabel->setEnabled(TRUE);
    algvariables->setEnabled(TRUE); 
    algvariableslabel->setEnabled(TRUE);
  }
  else {
    executebuttongroup->setEnabled(FALSE);
    zerobutton->setEnabled(FALSE);
    averagebutton->setEnabled(FALSE);
    continuebutton->setEnabled(FALSE);
    nonebutton->setEnabled(FALSE);
    algname->setEnabled(FALSE);
    algnamelabel->setEnabled(FALSE);
    executebuttongroup2->setEnabled(FALSE);
    contourbutton->setEnabled(FALSE);
    smoothbutton->setEnabled(FALSE);
    nonebutton2->setEnabled(FALSE);
    nameoftheexecution->setEnabled(FALSE);
    nameoftheexecutionlabel->setEnabled(FALSE); 
    algvariables->setEnabled(FALSE); 
    algvariableslabel->setEnabled(FALSE);
    threshold->setEnabled(FALSE);
    thresholdlabel->setEnabled(FALSE);
    weight1->setEnabled(FALSE);
    weight1label->setEnabled(FALSE);
    weight2->setEnabled(FALSE);
    weight2label->setEnabled(FALSE);
    weight3->setEnabled(FALSE);
    weight3label->setEnabled(FALSE);
    iterationflagline->setEnabled(FALSE);
    iterationflaglabel->setEnabled(FALSE);
  }
}*/

/** Activates contour and smooth fields fields 
@param void
@author Bruno M. Carvalho
@version 1.0 */
void executewindow::activateContourSmoothFields()
{
  if(contourbutton->isChecked() || smoothbutton->isChecked()) {
    threshold->setEnabled(TRUE);
    thresholdlabel->setEnabled(TRUE);
    weight1->setEnabled(TRUE);
    weight1label->setEnabled(TRUE);
    weight2->setEnabled(TRUE);
    weight2label->setEnabled(TRUE);
    weight3->setEnabled(TRUE);
    weight3label->setEnabled(TRUE);
    iterationflagline->setEnabled(TRUE);
    iterationflaglabel->setEnabled(TRUE);
  }
  else {
    threshold->setEnabled(FALSE);
    thresholdlabel->setEnabled(FALSE);
    weight1->setEnabled(FALSE);
    weight1label->setEnabled(FALSE);
    weight2->setEnabled(FALSE);
    weight2label->setEnabled(FALSE);
    weight3->setEnabled(FALSE);
    weight3label->setEnabled(FALSE);
    iterationflagline->setEnabled(FALSE);
    iterationflaglabel->setEnabled(FALSE);
  }
}

/** Deactivates contour and smooth fields fields 
@param void
@author Bruno M. Carvalho
@version 1.0 */
void executewindow::deactivateContourSmoothFields()
{
  if(nonebutton2->isChecked()) {
    threshold->setEnabled(FALSE);
    thresholdlabel->setEnabled(FALSE);
    weight1->setEnabled(FALSE);
    weight1label->setEnabled(FALSE);
    weight2->setEnabled(FALSE);
    weight2label->setEnabled(FALSE);
    weight3->setEnabled(FALSE);
    weight3label->setEnabled(FALSE);
    iterationflagline->setEnabled(FALSE);
    iterationflaglabel->setEnabled(FALSE);
  }
  else {
    threshold->setEnabled(TRUE);
    thresholdlabel->setEnabled(TRUE);
    weight1->setEnabled(TRUE);
    weight1label->setEnabled(TRUE);
    weight2->setEnabled(TRUE);
    weight2label->setEnabled(TRUE);
    weight3->setEnabled(TRUE);
    weight3label->setEnabled(TRUE);
    iterationflagline->setEnabled(TRUE);
    iterationflaglabel->setEnabled(TRUE);
  }
}

/** Returns a string for EXECUTE input sequence
@param void
@return execute
@author Bruno M. Carvalho
@version 1.0 */
QString executewindow::getOutput()
{
  QString output;

  output.sprintf("EXECUTE ");
  if(zerobutton->isChecked()) {
    output.append("ZERO ");
  }
  else
    if(averagebutton->isChecked()) {
      output.append("AVERAGE ");
    }
    else
      if(continuebutton->isChecked()) {
        output.append("CONTINUE ");
      }
      else
	if(phantombutton->isChecked()) {
	  output.append("PHANTOM ");
	}
  output+=algname->text();
  output.append(" ");
  if(contourbutton->isChecked()) {
    output.append("CONTOUR ");
  }
  else
    if(smoothbutton->isChecked()) {
      output.append("SMOOTH ");
    }
  output.append("\n");
  output+=nameoftheexecution->text();
//  if(strcmp(algvariables->text(),"")) {
//    output.append("\n");
//    output+=algvariables->text();
//  }
  //output.append("\n");
  if(contourbutton->isChecked() || smoothbutton->isChecked()) {
    output.append("\n");
    output+=threshold->text();
    output.append(" ");
    output+=weight1->text();
    output.append(" ");
    output+=weight2->text();
    output.append(" ");
    output+=weight3->text();
    output.append("\n");
    output+=iterationflagline->text();
    //output.append("\n");
  } 
  
    if(strcmp(algvariables->text(),"")) {
    output.append("\n");
    output+=algvariables->text();
  }
  return output;
}

void executewindow::checkValues()
{
  int f=0,cmp,alg;
  float t,w1,w2,w3;
  const char *str;
  char str1[5];

  str=(const char *)algname->text();
  str1[0]=stolower(str[0]);
  str1[1]=stolower(str[1]);
  str1[2]=stolower(str[2]);
  str1[3]=stolower(str[3]);
  str1[4]=0;
	//  cmp=strcmp(str1,"alg1");
	//cmp=strcmp(str1,"algp");     // SI bug 113 fix
	cmp=strcmp(str1,"alp1"); // bug 267, Ran
	if(!cmp) alg=1;
	else {
		//    cmp=strcmp(str1,"alg2");
		//cmp=strcmp(str1,"algb");   // SI bug 113 fix
		cmp=strcmp(str1,"alb1"); // bug 267, Ran
		if(!cmp) alg=2;
		else {
			cmp=strcmp(str1,"alts");
			if(!cmp) alg=3;
			else {
				cmp=strcmp(str1,"art");
				if(!cmp) alg=4;
				else {
					cmp=strcmp(str1,"back");
					if(!cmp) alg=5;
					else {
						cmp=strcmp(str1,"conv");
						if(!cmp) alg=6;
						else {
							cmp=strcmp(str1,"dcon");
							if(!cmp) alg=7;
							else {
								cmp=strcmp(str1,"emap");
								if(!cmp) alg=8;
								else {
									cmp=strcmp(str1,"four");
									if(!cmp) alg=9;
									else {
										cmp=strcmp(str1,"lino");
										if(!cmp) alg=10;
										else {
											cmp=strcmp(str1,"mart");
											if(!cmp) alg=11;
											else {
												cmp=strcmp(str1,"mspb");
												if(!cmp) alg=12;
												else {
													cmp=strcmp(str1,"quad");
													if(!cmp) alg=13;
													else {
														cmp=strcmp(str1,"rfl");
														if(!cmp) alg=14;
														else {
															cmp=strcmp(str1,"sirt");
															if(!cmp) alg=15;
															else {
																cmp=strcmp(str1,"alp2");
																if(!cmp) alg=16;
																else {
																	cmp=strcmp(str1,"alp3");
																	if(!cmp) alg=17;
																	else {
																		cmp=strcmp(str1,"alp4");
																		if(!cmp) alg=18;
																		else {
																			cmp=strcmp(str1,"alp5");
																			if(!cmp) alg=19;
																			else {
																				cmp=strcmp(str1,"alb2");
																				if(!cmp) alg=20;
																				else {
																					cmp=strcmp(str1,"alb3");
																					if(!cmp) alg=21;
																					else {
																						cmp=strcmp(str1,"alb4");
																						if(!cmp) alg=22;
																						else {
																							cmp=strcmp(str1,"alb5");
																							if(!cmp) alg=23;
																							else {
																								cmp=strcmp(str1,"sart");
																								if(!cmp) alg=23;
																								else f=1;
																							}
																						}
																					}
																				}
																			}
																		}
																	}
																}
															}
														}
													}
												}
											}
										}
									}
								}
							}
						}
					}
				}
			}
		}
	}
  if(contourbutton->isChecked() || smoothbutton->isChecked()) {
    t=strtof((const char *)threshold->text());
    if(errnum) 
      f=2;
    w1=strtof((const char *)weight1->text());
    if(errnum) 
      f=3;
    w2=strtof((const char *)weight2->text());
    if(errnum) 
      f=4;
    w3=strtof((const char *)weight3->text());
    if(errnum) 
      f=5;
    if(!isintspace(iterationflagline->text()))
      f=6;
  }

  switch(f) {
  case 0: accept();
    break;
  case 1: QMessageBox::information(this,"SnarkInput","Error!\n"
                                   "Invalid algorithm name.Algorithm name must be one of:\n"
  // SI bug 113 fix  alg1 -> algp ; alg2 -> algb
								   // "ALG1, ALG2, ALTSCHULER, ART, BACKPROJECTION,\n"
  // Ran, bug 267  added ALP1 - ALP5 and ALB1 - ALB5
								   //"ALGP, ALGB, ALTSCHULER, ART, BACKPROJECTION,\n"
								   "ALP1, ALP2, ALP3, ALP4, ALP5, ALB1, ALB2, \n"
								   "ALB3, ALB4, ALB5, ALTSCHULER, ART, \n"
								   "BACKPROJECTION,CONVOLUTION, DCONV, EMAP,\n"
                                   "FOURIER, LINO, MART, MSPB, QUAD, RFL, SART or SIRT");    break;
  case 2: QMessageBox::information(this,"SnarkInput","Error!\n"
                                   "Invalid threshold format.");
          threshold->setFocus();
    break;
  case 3: QMessageBox::information(this,"SnarkInput","Error!\n"
                                   "Invalid weight1 format.");
          weight1->setFocus();
    break;
  case 4: QMessageBox::information(this,"SnarkInput","Error!\n"
                                   "Invalid weight2 format.");
          weight2->setFocus();
    break;
  case 5: QMessageBox::information(this,"SnarkInput","Error!\n"
                                   "Invalid weight3 format.");
          weight3->setFocus();
    break;
  case 6: QMessageBox::information(this,"SnarkInput","Error!\n"
                                   "iteration_flag_line must contain\n"
                                   "only 0-9 and spaces and must\n"
                                   "be non-empty");
          iterationflagline->setFocus();
    break;
  }
}
